@@ -1,5 +1,5 @@
 ;; dump.jl -- dumping of Lisp forms to C code
-;;  $Id: dump.jl,v 1.16 2000/03/04 23:34:57 john Exp $
+;;  $Id: dump.jl,v 1.19 2000/06/19 15:53:54 john Exp $
 
 ;;  Copyright (C) 1998,1999 John Harper <john@dcs.warwick.ac.uk>
 
@@ -21,6 +21,8 @@
 
 (provide 'dump)
 
+;; XXX don't use this, it doesn't grok the module system changes
+
 ;; Commentary:
 
 ;; Lisp modules generally consist almost wholly of constant
@@ -239,15 +241,16 @@ Note that each input file will be loaded from the lisp-lib-directory with
     (assoc tag (nthcdr 2 cell))))
 
 ;; Add a pair (TAG . VALUE) to be associated with CELL
-(defun dump-add-state (cell tag value &aux pair)
-  (if (setq pair (assoc tag (nthcdr 2 cell)))
-      (rplacd pair value)
-    (rplacd (nthcdr 1 cell) (cons (cons tag value) (nthcdr 2 cell)))))
+(defun dump-add-state (cell tag value)
+  (let ((pair (assoc tag (nthcdr 2 cell))))
+    (if pair
+	(rplacd pair value)
+      (rplacd (nthcdr 1 cell) (cons (cons tag value) (nthcdr 2 cell))))))
 
 ;; Add a pair (PROP . VALUE) to the plist state of CELL. Currently this
 ;; doesn't handle PROP already being in the list, it just pushes another
 ;; pair on the head
-(defun dump-state-put (cell prop value &aux plist tem)
+(defun dump-state-put (cell prop value)
   (dump-add-state cell 'plist
 		  (cons prop (cons value (dump-get-state cell 'plist)))))
 
@@ -363,9 +366,11 @@ Note that each input file will be loaded from the lisp-lib-directory with
 ;; For all symbol cells in LIST that have a plist property, add its value
 ;; as a constant
 (defun dump-fix-plists (lst)
-  (mapc (lambda (x &aux plist)
-	  (when (setq plist (dump-has-state-p x 'plist))
-	    (rplacd plist (dump-get-label (dump-add-constant (cdr plist))))))
+  (mapc (lambda (x)
+	  (let ((plist (dump-has-state-p x 'plist)))
+	    (when plist
+	      (rplacd plist (dump-get-label
+			     (dump-add-constant (cdr plist)))))))
 	lst))
 
 
@@ -526,7 +531,7 @@ Note that each input file will be loaded from the lisp-lib-directory with
 	       (label (dump-get-label cell)))
 	    (@ "const u_char %s_data[] = %S;\n" label string)
 	    (@ "const const_rep_string %s = {\n" label)
-	    (@ "  0x%x,\n" (logior (ash (length string) 8) 0x45))
+	    (@ "  0x%x,\n" (logior (ash (length string) 8) #x45))
 	    (@ "  rep_VAL(%s_data)\n" label)
 	    (@ "};\n\n"))) head))
     
@@ -567,7 +572,7 @@ Note that each input file will be loaded from the lisp-lib-directory with
 	       (plist (dump-get-state cell 'plist)))
 	    (@ "/* %s */\n" symbol)
 	    (@ "rep_symbol %s = {\n" label)
-	    (@ "  0x%x,\n" (logior 0x41
+	    (@ "  0x%x,\n" (logior #x41
 				   (if is-constant (ash 1 (+ 8 0)) 0)
 				   (if is-defined  (ash 1 (+ 8 7)) 0)
 				   (if is-special  (ash 1 (+ 8 4)) 0)))
@@ -600,7 +605,7 @@ Note that each input file will be loaded from the lisp-lib-directory with
   ;; Output to vector cells in the list HEAD, TYPE should be vector or bytecode
   (defun dump-output-vectors (head type)
     (let
-	((type-value (if (eq type 'vector) 0x43 0x47)))
+	((type-value (if (eq type 'vector) #x43 #x47)))
       (@ "\n\f\n/* Constant %ss */\n\n" type)
       (mapc (lambda (cell)
 	      (let
