@@ -1,5 +1,5 @@
 # rules.mk.sh -- Build dynamically-loadable objects for librep
-# $Id: rules.mk.sh,v 1.5 2000/02/02 14:02:15 john Exp $
+# $Id: rules.mk.sh,v 1.6 2000/07/20 11:00:32 john Exp $
 
 repdir=$1
 repexecdir=$2
@@ -10,6 +10,7 @@ cat <<EOF
 
 repdir=$repdir
 repexecdir=$repexecdir
+rpath_repexecdir=$repexecdir
 repdocfile=$repdocfile
 
 rep_LIBTOOL=\$(repexecdir)/libtool
@@ -18,12 +19,14 @@ rep_LIBTOOL=\$(repexecdir)/libtool
 # foo.la : foo.lo bar.lo
 #	\$(rep_DL_LD) link-opts...
 
-rep_DL_LD=\$(rep_LIBTOOL) \$(CC) -avoid-version -module -rpath \$(repexecdir)
+rep_DL_LD=\$(rep_LIBTOOL) --mode=link \$(CC) -avoid-version -module \
+	  -rpath \$(rpath_repexecdir)
+
 rep_DL_INSTALL=\$(rep_LIBTOOL) \$(INSTALL)
 rep_DL_UNINSTALL=\$(rep_LIBTOOL) rm
 
 # Rule for libtool controlled C objects
 %.lo : %.c
-	\$(rep_LIBTOOL) \$(CC) -c \$(CPPFLAGS) \$(CFLAGS) \$<
+	\$(rep_LIBTOOL) --mode=compile \$(CC) -c \$(CPPFLAGS) \$(CFLAGS) \$<
 
 EOF
