@@ -1,6 +1,6 @@
 /* bytecodes.h -- Constant definitions of lispmach byte-codes
    Copyright (C) 1993, 1994 John Harper <john@dcs.warwick.ac.uk>
-   $Id: bytecodes.h,v 1.22 2000/02/27 13:16:59 john Exp $
+   $Id: bytecodes.h,v 1.32 2000/07/09 17:38:45 john Exp $
 
    This file is part of Jade.
 
@@ -21,7 +21,7 @@
 #ifndef BYTECODES_H
 #define BYTECODES_H
 
-#define BYTECODE_MAJOR_VERSION 9
+#define BYTECODE_MAJOR_VERSION 10
 #define BYTECODE_MINOR_VERSION 0
 
 /* Number of bits encoded in each extra opcode forming the argument. */
@@ -61,13 +61,18 @@
 /* Push the list formed from the top ARG values on the stack. */
 #define OP_LIST 0x28
 
-/* Lexically bind the value on the top of the stack to the symbol
-   const[ARG]. Pops the value off the stack. */
-#define OP_BIND 0x30
-
 /* Pushes the ARG'th value in the lexical environment */
 #define OP_REFN 0x38
 
+#define OP_REFN_0 0x38
+#define OP_REFN_1 0x39
+#define OP_REFN_2 0x3a
+#define OP_REFN_3 0x3b
+#define OP_REFN_4 0x3c
+#define OP_REFN_5 0x3d
+#define OP_REFN_6 0x3e
+#define OP_REFN_7 0x3f
+
 /* Sets the ARG'th value in the lexical environment. Pops value */
 #define OP_SETN 0xe8
 
@@ -87,7 +92,7 @@
 
 #define OP_REF 0x40			/* push (symbol-value pop) */
 #define OP_SET 0x41			/* call-2 set */
-#define OP_DSET 0x42			/* call-2 set & mark as defvar'd */
+#define OP_FLUID_REF 0x42		/* call-1 fluid-ref */
 #define OP_ENCLOSE 0x43			/* push (make-closure pop[1] nil) */
 #define OP_INIT_BIND 0x44		/* new-binding-set */
 #define OP_UNBIND 0x45			/* rewind-binding-set */
@@ -107,7 +112,7 @@
 #define OP_ASET 0x52			/* call-3 aset */
 #define OP_AREF 0x53			/* call-2 aref */
 #define OP_LENGTH 0x54			/* call-1 length */
-#define OP_EVAL 0x55			/* call-1 eval */
+#define OP_BIND 0x55
 #define OP_ADD 0x56			/* push (+ pop[1] pop[2]) */
 #define OP_NEG 0x57			/* push (- pop[1]) */
 #define OP_SUB 0x58			/* push (- pop[1] pop[2]) */
@@ -120,8 +125,8 @@
 #define OP_LAND 0x5f			/* push (logand pop[1] pop[2]) */
 #define OP_EQUAL 0x60			/* push (equal pop[1] pop[2]) */
 #define OP_EQ 0x61			/* push (eq pop[1] pop[2]) */
-#define OP_UNUSED1 0x62
-#define OP_UNUSED2 0x63
+#define OP_STRUCT_REF 0x62		/* push (structure-ref pop[1] pop[2])*/
+#define OP_SCM_TEST 0x63
 #define OP_GT 0x64			/* push (> pop[1] pop[2]) */
 #define OP_GE 0x65			/* push (>= pop[1] pop[2]) */
 #define OP_LT 0x66			/* push (< pop[1] pop[2]) */
@@ -209,7 +214,7 @@
 #define OP_TRUNCATE 0xae
 #define OP_ROUND 0xaf
 
-#define OP_BINDOBJ 0xb0			/* bind stk[0] */
+#define OP_UNUSED1 0xb0
 #define OP_FORBID 0xb1
 #define OP_PERMIT 0xb2
 
@@ -227,8 +232,18 @@
 #define OP_MOD 0xbb			/* push (mod pop[1] pop[2]) */
 
 #define OP_MAKE_CLOSURE 0xbc		/* push (make-closure pop[1] pop[2]) */
+#define OP_UNBINDALL_0 0xbd
 #define OP_CLOSUREP 0xbe		/* push (closurep pop[1]) */
-#define OP_BINDENV 0xbf
+#define OP_POP_ALL 0xbf
+
+#define OP_FLUID_SET 0xc0
+#define OP_FLUID_BIND 0xc1
+
+#define OP_MEMQL 0xc2			/* call-2 memql */
+#define OP_NUM_EQ 0xc3
+
+#define OP_TEST_SCM 0xc4
+#define OP_TEST_SCM_F 0xc5
 
 
 /* Jump opcodes */
