@@ -1,6 +1,6 @@
 /* continuations.c -- continuations, much stack hackery..
    Copyright (C) 2000 John Harper <john@dcs.warwick.ac.uk>
-   $Id: continuations.c,v 1.18 2000/04/06 21:46:58 john Exp $
+   $Id: continuations.c,v 1.28 2000/07/15 11:41:35 john Exp $
 
    This file is part of librep.
 
@@ -129,6 +129,20 @@ char *alloca ();
 # include <sys/time.h>
 #endif
 
+#if defined (DEBUG)
+# define DB(x) printf x
+#else
+# define DB(x)
+#endif
+
+/* Threads only preempted when this is zero. */
+int rep_thread_lock = 0;
+
+/* True when the current thread should be preempted soon */
+rep_bool rep_pending_thread_yield;
+
+#ifdef WITH_CONTINUATIONS
+
 #if STACK_DIRECTION == 0
 # error "stack growth direction unknown"
 #elif STACK_DIRECTION > 0
@@ -154,12 +168,6 @@ char *alloca ();
 # endif
 #endif
 
-#if defined (DEBUG)
-# define DB(x) printf x
-#else
-# define DB(x)
-#endif
-
 typedef struct rep_barrier_struct rep_barrier;
 typedef struct rep_continuation_struct rep_continuation;
 typedef struct rep_thread_struct rep_thread;
@@ -237,10 +245,12 @@ struct rep_thread_struct {
     rep_thread *next, *pred;
     repv name;
     rep_continuation *cont;
-    repv env, special_env, fh_env;
-    repv (*bytecode)(repv, repv, repv, repv);
+    repv env, structure;
     int lock;
     struct timeval run_at;
+    rep_bool (*poll)(rep_thread *t, repv arg);
+    repv poll_arg;
+    repv exit_val;
 };
 
 #define XTHREADP(v)	rep_CELL16_TYPEP(v, rep_thread_type)
@@ -253,12 +263,6 @@ struct rep_thread_struct {
 static int rep_thread_type;
 static rep_thread *threads;
 
-/* Threads only preempted when this is zero. */
-int rep_thread_lock = 0;
-
-/* True when the current thread should be preempted soon */
-rep_bool rep_pending_thread_yield;
-
 #define TV_LATER_P(t1, t2)			\
     (((t1)->tv_sec > (t2)->tv_sec)		\
      || (((t1)->tv_sec == (t2)->tv_sec)		\
@@ -516,8 +520,8 @@ primitive_invoke_continuation (rep_continuation *c, repv ret)
     anc = common_ancestor (barriers, dest_hist, depth);
     if (anc == 0)
     {
-	Fsignal (Qerror,
-		 rep_LIST_1 (rep_string_dup ("unreachable continuation")));
+	DEFSTRING (unreachable, "unreachable continuation");
+	Fsignal (Qerror, rep_LIST_1 (rep_VAL (&unreachable)));
 	return;
     }
 
@@ -560,8 +564,8 @@ DEFUN("primitive-invoke-continuation", Fprimitive_invoke_continuation,
     if (cont == rep_NULL || !rep_CONTINP(cont)
 	|| (rep_CONTIN(cont)->car & CF_INVALID))
     {
-	return Fsignal (Qerror, rep_LIST_1 (rep_string_dup
-					    ("invalid continuation")));
+	DEFSTRING (invalid, "invalid continuation");
+	return Fsignal (Qerror, rep_LIST_1 (rep_VAL (&invalid)));
     }
 
     primitive_invoke_continuation (rep_CONTIN (cont), ret);
@@ -620,7 +624,10 @@ primitive_call_cc (repv (*callback)(rep_continuation *, void *), void *data)
     repv ret;
 
     if (root_barrier == 0)
-	return Fsignal (Qerror, rep_LIST_1 (rep_string_dup ("No dynamic root")));
+    {
+	DEFSTRING (no_root, "no dynamic root");
+	return Fsignal (Qerror, rep_LIST_1 (rep_VAL (&no_root)));
+    }
 
     c = rep_ALLOC_CELL (sizeof (rep_continuation));
     rep_data_after_gc += sizeof (rep_continuation);
@@ -720,8 +727,9 @@ inner_call_cc (rep_continuation *c, void *data)
 {
     repv proxy;
     proxy = Fmake_closure (rep_VAL(&Sprimitive_invoke_continuation), Qnil);
-    rep_FUNARG(proxy)->env = Fcons (Fcons (Qcontinuation, rep_VAL(c)),
-				    rep_FUNARG(proxy)->env);
+    rep_FUNARG(proxy)->env
+	= rep_add_binding_to_env (rep_FUNARG(proxy)->env,
+				  Qcontinuation, rep_VAL(c));
     return rep_call_lisp1 ((repv) data, proxy);
 }
 
@@ -746,18 +754,14 @@ static inline void
 thread_save_environ (rep_thread *t)
 {
     t->env = rep_env;
-    t->special_env = rep_special_env;
-    t->fh_env = rep_fh_env;
-    t->bytecode = rep_bytecode_interpreter;
+    t->structure = rep_structure;
 }
 
 static inline void
 thread_load_environ (rep_thread *t)
 {
     rep_env = t->env;
-    rep_special_env = t->special_env;
-    rep_fh_env = t->fh_env;
-    rep_bytecode_interpreter = t->bytecode;
+    rep_structure = t->structure;
 }
 
 static void
@@ -839,6 +843,23 @@ thread_wake (rep_thread *t)
     enqueue_thread (t, root);
 }
 
+static rep_bool
+poll_threads (rep_barrier *root)
+{
+    rep_bool woke_any = rep_FALSE;
+    rep_thread *t, *next;
+    for (t = root->susp_head; t != 0; t = next)
+    {
+	next = t->next;
+	if (t->poll && t->poll (t, t->poll_arg))
+	{
+	    thread_wake (t);
+	    woke_any = rep_TRUE;
+	}
+    }
+    return woke_any;
+}
+
 static repv
 inner_thread_invoke (rep_continuation *c, void *data)
 {
@@ -892,6 +913,11 @@ again:
 	    DB (("no more threads, throwing to root..\n"));
 	    return;
 	}
+	else if (poll_threads (root_barrier))
+	{
+	    /* something woke */
+	    goto again;
+	}
 	else
 	{
 	    rep_thread *b = root_barrier->susp_head;
@@ -946,6 +972,9 @@ new_thread (repv name)
     memset (t, 0, sizeof (rep_thread));
     t->car = rep_thread_type;
     t->name = name;
+    t->poll = 0;
+    t->poll_arg = Qnil;
+    t->exit_val = rep_NULL;
     t->next_alloc = threads;
     threads = t;
     return t;
@@ -987,6 +1016,7 @@ make_thread (repv thunk, repv name)
 	t->car |= TF_EXITED;
 	if (ret != rep_NULL)
 	{
+	    t->exit_val = ret;
 	    thread_delete (t);
 	    thread_invoke ();
 	    assert (rep_throw_value == exit_barrier_cell);
@@ -1027,17 +1057,17 @@ thread_yield (void)
 	root_barrier->tail = old;
     }
 
-    /* check suspend queue for threads whose time is up */
+    /* check suspend queue for threads that need waking */
+
     if (root_barrier->susp_head != 0)
 	gettimeofday (&now, 0);
     for (ptr = root_barrier->susp_head; ptr != 0; ptr = next)
     {
 	next = ptr->next;
-	if (TV_LATER_P (&now, &ptr->run_at))
+	if (TV_LATER_P (&now, &ptr->run_at)
+	    || (ptr->poll && ptr->poll (ptr, ptr->poll_arg)))
 	{
-	    unlink_thread (ptr);
-	    ptr->car &= ~TF_SUSPENDED;
-	    enqueue_thread (ptr, root_barrier);
+	    thread_wake (ptr);
 	}
     }
 
@@ -1051,7 +1081,8 @@ thread_yield (void)
 }
 
 static void
-thread_suspend (rep_thread *t, u_long msecs)
+thread_suspend (rep_thread *t, u_long msecs,
+		rep_bool (*poll)(rep_thread *t, repv arg), repv poll_arg)
 {
     rep_barrier *root = t->cont->root;
     assert (!(t->car & TF_SUSPENDED));
@@ -1077,6 +1108,8 @@ thread_suspend (rep_thread *t, u_long msecs)
 	    t->run_at.tv_usec = t->run_at.tv_usec % 1000000;
 	}
     }
+    t->poll = poll;
+    t->poll_arg = poll_arg;
     enqueue_thread (t, root);
     if (root_barrier->active == t)
 	thread_invoke ();
@@ -1100,6 +1133,7 @@ rep_max_sleep_for (void)
     else if (root->susp_head != 0)
     {
 	/* other threads sleeping, how long until the first wakes? */
+	/* XXX ignores polling */
 	struct timeval now;
 	long msecs;
 	gettimeofday (&now, 0);
@@ -1166,8 +1200,7 @@ mark_cont (repv obj)
 	rep_MARKVAL(lc->fun);
 	rep_MARKVAL(lc->args);
 	rep_MARKVAL(lc->saved_env);
-	rep_MARKVAL(lc->saved_special_env);
-	rep_MARKVAL(lc->saved_fh_env);
+	rep_MARKVAL(lc->saved_structure);
     }
     for (matches = c->regexp_data;
 	 matches != 0 && !SP_OLDER_P ((char *) matches, c->stack_bottom);
@@ -1175,6 +1208,7 @@ mark_cont (repv obj)
     {
 	struct rep_saved_regexp_data *sd
 	    = FIXUP(struct rep_saved_regexp_data *, c, matches);
+	assert (sd->type ==  rep_reg_obj || sd->type == rep_reg_string);
 	if(sd->type == rep_reg_obj)
 	{
 	    int i;
@@ -1237,9 +1271,10 @@ mark_thread (repv obj)
 {
     rep_MARKVAL (rep_VAL (THREAD (obj)->cont));
     rep_MARKVAL (THREAD (obj)->env);
-    rep_MARKVAL (THREAD (obj)->special_env);
-    rep_MARKVAL (THREAD (obj)->fh_env);
+    rep_MARKVAL (THREAD (obj)->structure);
     rep_MARKVAL (THREAD (obj)->name);
+    rep_MARKVAL (THREAD (obj)->poll_arg);
+    rep_MARKVAL (THREAD (obj)->exit_val);
 }
 
 static void
@@ -1274,15 +1309,80 @@ print_thread (repv stream, repv obj)
     rep_stream_putc (stream, '>');
 }
 
+#else /* WITH_CONTINUATIONS */
+
+repv
+rep_call_with_barrier (repv (*callback)(repv), repv arg,
+		       rep_bool closed, void (*in)(void *),
+		       void (*out)(void *), void *data)
+{
+    return callback (arg);
+}
+
+DEFSTRING (ccc_missing, "call/cc was not included in this system");
+
+static repv
+call_cc_missing (void)
+{
+    return Fsignal (Qerror, rep_LIST_1 (rep_VAL (&ccc_missing)));
+}
+
+
+DEFUN ("call/cc", Fcall_cc, Scall_cc, (repv fun), rep_Subr1)
+{
+    return call_cc_missing ();
+}
+
+DEFUN("continuation-callable-p", Fcontinuation_callable_p,
+      Scontinuation_callable_p, (repv cont), rep_Subr1)
+{
+    return rep_signal_arg_error (cont, 1);
+}
+
+u_long
+rep_max_sleep_for (void)
+{
+    return UINT_MAX;
+}
+
+#endif /* !WITH_CONTINUATIONS */
+
 
 /* misc lisp functions */
 
+/* Bind one object, returning the handle to later unbind by. */
+static repv
+bind_object(repv obj)
+{
+    rep_type *t = rep_get_data_type(rep_TYPE(obj));
+    if (t->bind != 0)
+	return t->bind(obj);
+    else
+	return Qnil;
+}
+
+static void
+unbind_object (repv handle)
+{
+    repv obj;
+    rep_type *t;
+    if (handle == Qnil)
+	return;
+    else if (rep_CONSP (handle))
+	obj = rep_CAR (handle);
+    else
+	obj = handle;
+    t = rep_get_data_type (rep_TYPE (obj));
+    if (t->unbind != 0)
+	t->unbind(handle);
+}
+
 static void
 call_with_inwards (void *data_)
 {
     repv *data = data_;
     if (data[0] != rep_NULL)
-	data[1] = rep_bind_object (data[0]);
+	data[1] = bind_object (data[0]);
     else
 	data[1] = rep_NULL;
 }
@@ -1293,7 +1393,7 @@ call_with_outwards (void *data_)
     repv *data = data_;
     if (data[1] != rep_NULL)
     {
-	rep_unbind_object (data[1]);
+	unbind_object (data[1]);
 	data[1] = rep_NULL;
     }
 }
@@ -1314,7 +1414,7 @@ unbound. If THUNK is subsequently reentered, ARG will be rebound.
 {
     repv data[2];			/* { ARG, HANDLE } */
     data[0] = arg;
-    data[1] = rep_bind_object(data[0]);
+    data[1] = bind_object(data[0]);
     if (data[1] != rep_NULL)
     {
 	repv ret;
@@ -1323,7 +1423,7 @@ unbound. If THUNK is subsequently reentered, ARG will be rebound.
 	ret = rep_call_with_barrier (rep_call_lisp0, thunk,
 				     rep_FALSE, call_with_inwards,
 				     call_with_outwards, data);
-	rep_unbind_object (data[1]);
+	unbind_object (data[1]);
 	rep_POPGCN;
 	return ret;
     }
@@ -1399,7 +1499,11 @@ new thread will begin by calling THUNK, a function with zero
 parameters.
 ::end:: */
 {
+#ifdef WITH_CONTINUATIONS
     return rep_VAL (make_thread (thunk, name));
+#else
+    return call_cc_missing ();
+#endif
 }
 
 DEFUN("thread-yield", Fthread_yield, Sthread_yield, (void), rep_Subr0) /*
@@ -1410,7 +1514,11 @@ Pass control away from the current thread if other threads are waiting
 to run.
 ::end:: */
 {
+#ifdef WITH_CONTINUATIONS
     return thread_yield () ? Qt : Qnil;
+#else
+    return Qnil;
+#endif
 }
 
 DEFUN("thread-delete", Fthread_delete, Sthread_delete, (repv th), rep_Subr1) /*
@@ -1423,11 +1531,15 @@ will be passed to the next runnable thread. Deleting the last runnable
 thread results forces the containing dynamic root to be closed.
 ::end:: */
 {
+#ifdef WITH_CONTINUATIONS
     if (th == Qnil)
 	th = Fcurrent_thread (Qnil);
     rep_DECLARE1 (th, THREADP);
     thread_delete (THREAD (th));
     return Qnil;
+#else
+    return rep_signal_arg_error (th, 1);
+#endif
 }
 
 DEFUN("thread-suspend", Fthread_suspend,
@@ -1441,11 +1553,54 @@ thread will pass control to the next runnable thread. If there are no
 runnable threads, then sleep until the next thread becomes runnable.
 ::end:: */
 {
+#ifdef WITH_CONTINUATIONS
     if (th == Qnil)
 	th = Fcurrent_thread (Qnil);
     rep_DECLARE1 (th, THREADP);
-    thread_suspend (THREAD (th), rep_INTP (msecs) ? rep_INT (msecs) : 0);
+    thread_suspend (THREAD (th), rep_INTP (msecs) ? rep_INT (msecs) : 0,
+		    0, Qnil);
     return Qnil;
+#else
+    return rep_signal_arg_error (th, 1);
+#endif
+}
+
+#ifdef WITH_CONTINUATIONS
+static rep_bool
+thread_join_poller (rep_thread *t, repv arg)
+{
+    return (THREAD (arg)->car & TF_EXITED) ? rep_TRUE : rep_FALSE;
+}
+#endif
+
+DEFUN("thread-join", Fthread_join,
+      Sthread_join, (repv th, repv msecs, repv def), rep_Subr3) /*
+::doc:thread-join::
+thread-join THREAD [MSECS] [DEFAULT-VALUE]
+
+Suspend the current thread until THREAD has exited, or MSECS
+milliseconds have passed. If THREAD exits normally, return the value of
+the last form it evaluated, else return DEFAULT-VALUE.
+::end:: */
+{
+#ifdef WITH_CONTINUATIONS
+    repv self = Fcurrent_thread (Qnil);
+    rep_DECLARE (1, th, XTHREADP (th) && th != self);
+    if (THREADP (self))
+    {
+	rep_GC_root gc_th;
+	rep_PUSHGC (gc_th, th);
+	thread_suspend (THREAD (self),
+			rep_INTP (msecs) ? rep_INT (msecs) : 0,
+			thread_join_poller, th);
+	rep_POPGC;
+	if ((THREAD (th)->car & TF_EXITED) && THREAD (th)->exit_val)
+	    return THREAD (th)->exit_val;
+    }
+    return def;
+#else
+    return rep_signal_arg_error (th, 1);
+#endif
 }
 
 DEFUN("thread-wake", Fthread_wake, Sthread_wake, (repv th), rep_Subr1) /*
@@ -1456,11 +1611,15 @@ If THREAD (or the current thread) is currently suspended, mark it as
 being runnable once more.
 ::end:: */
 {
+#ifdef WITH_CONTINUATIONS
     if (th == Qnil)
 	th = Fcurrent_thread (Qnil);
     rep_DECLARE1 (th, THREADP);
     thread_wake (THREAD (th));
     return Qnil;
+#else
+    return rep_signal_arg_error (th, 1);
+#endif
 }
 
 DEFUN("threadp", Fthreadp, Sthreadp, (repv arg), rep_Subr1) /*
@@ -1470,7 +1629,11 @@ threadp ARG
 Return `t' if ARG is a thread object.
 ::end:: */
 {
+#ifdef WITH_CONTINUATIONS
     return XTHREADP (arg) ? Qt : Qnil;
+#else
+    return Qnil;
+#endif
 }
 
 DEFUN("thread-suspended-p", Fthread_suspended_p,
@@ -1481,8 +1644,12 @@ thread-suspended-p THREAD
 Return `t' if THREAD is currently suspended from running.
 ::end:: */
 {
+#ifdef WITH_CONTINUATIONS
     rep_DECLARE1 (th, THREADP);
     return (THREAD (th)->car & TF_SUSPENDED) ? Qt : Qnil;
+#else
+    return rep_signal_arg_error (th, 1);
+#endif
 }
 
 DEFUN("thread-exited-p", Fthread_exited_p,
@@ -1493,8 +1660,12 @@ thread-exited-p THREAD
 Return `t' if THREAD has exited.
 ::end:: */
 {
+#ifdef WITH_CONTINUATIONS
     rep_DECLARE1 (th, XTHREADP);
     return (THREAD (th)->car & TF_EXITED) ? Qt : Qnil;
+#else
+    return rep_signal_arg_error (th, 1);
+#endif
 }
 
 DEFUN("current-thread", Fcurrent_thread,
@@ -1506,12 +1677,16 @@ Return the currently executing thread, or `nil' if threaded execution
 is not currently taking place.
 ::end:: */
 {
+#ifdef WITH_CONTINUATIONS
     rep_barrier *root = get_dynamic_root (rep_INTP (depth_)
 					  ? rep_INT (depth_) : 0);
     if (root == 0)
 	return Qnil;
     else
 	return (root->active) ? rep_VAL (root->active) : Qnil;
+#else
+    return Qnil;
+#endif
 }
 
 DEFUN("all-threads", Fall_threads, Sall_threads, (repv depth_), rep_Subr1) /*
@@ -1521,6 +1696,7 @@ all-threads [DEPTH]
 Return a list of all threads.
 ::end:: */
 {
+#ifdef WITH_CONTINUATIONS
     rep_barrier *root = get_dynamic_root (rep_INTP (depth_)
 					  ? rep_INT (depth_) : 0);
     if (root == 0)
@@ -1535,6 +1711,9 @@ Return a list of all threads.
 	    out = Fcons (rep_VAL (ptr), out);
 	return out;
     }
+#else
+    return Qnil;
+#endif
 }
 
 DEFUN("thread-forbid", Fthread_forbid, Sthread_forbid, (void), rep_Subr0) /*
@@ -1570,8 +1749,12 @@ thread-name THREAD
 Return the name of the thread THREAD.
 ::end:: */
 {
+#ifdef WITH_CONTINUATIONS
     rep_DECLARE1 (th, XTHREADP);
     return THREAD (th)->name;
+#else
+    return rep_signal_arg_error (th, 1);
+#endif
 }
 
 
@@ -1580,6 +1763,7 @@ Return the name of the thread THREAD.
 void
 rep_continuations_init (void)
 {
+#ifdef WITH_CONTINUATIONS
     rep_continuation_type = rep_register_new_type ("continuation",
 						   rep_ptr_cmp,
 						   print_cont, print_cont, 
@@ -1589,9 +1773,13 @@ rep_continuations_init (void)
 					     print_thread, print_thread, 
 					     sweep_thread, mark_thread,
 					     0, 0, 0, 0, 0, 0, 0);
+    exit_barrier_cell = Fcons (Qnil, Qnil);
+    rep_mark_static (&exit_barrier_cell);
     rep_INTERN(continuation);
-    rep_ADD_SUBR(Scall_cc);
     rep_ADD_SUBR(Sprimitive_invoke_continuation);
+#endif
+
+    rep_ADD_SUBR(Scall_cc);
     rep_ADD_SUBR(Scontinuation_callable_p);
     rep_ADD_SUBR(Scall_with_object);
     rep_ADD_SUBR(Scall_with_dynamic_root);
@@ -1600,6 +1788,7 @@ rep_continuations_init (void)
     rep_ADD_SUBR(Sthread_yield);
     rep_ADD_SUBR(Sthread_delete);
     rep_ADD_SUBR(Sthread_suspend);
+    rep_ADD_SUBR(Sthread_join);
     rep_ADD_SUBR(Sthread_wake);
     rep_ADD_SUBR(Sthreadp);
     rep_ADD_SUBR(Sthread_suspended_p);
@@ -1609,6 +1798,4 @@ rep_continuations_init (void)
     rep_ADD_SUBR(Sthread_forbid);
     rep_ADD_SUBR(Sthread_permit);
     rep_ADD_SUBR(Sthread_name);
-    exit_barrier_cell = Fcons (Qnil, Qnil);
-    rep_mark_static (&exit_barrier_cell);
 }
