@@ -1,6 +1,6 @@
 /* files.c -- Extendable file handling
    Copyright (C) 1998 John Harper <john@dcs.warwick.ac.uk>
-   $Id: files.c,v 1.24 2000/02/13 22:15:48 john Exp $
+   $Id: files.c,v 1.27 2000/07/14 11:08:37 john Exp $
 
    This file is part of Jade.
 
@@ -148,6 +148,8 @@ DEFSYM(read, "read");
 DEFSYM(write, "write");
 DEFSYM(append, "append");
 
+DEFSYM(fh_env_key, "fh-env-key");
+
 /* Vector of blocked operations */
 struct blocked_op *rep_blocked_ops[op_MAX];
 
@@ -157,6 +159,13 @@ int rep_op_insert_file_contents = op_insert_file_contents;
 
 
 
+static inline repv
+get_fh_env (void)
+{
+    repv ret = F_structure_ref (rep_structure, Qfh_env_key);
+    return rep_VOIDP (ret) ? Qt : ret;
+}
+
 repv
 rep_signal_file_error(repv cdr)
 {
@@ -250,11 +259,15 @@ rep_call_file_handler(repv handler, int op, repv sym, int nargs, ...)
 
     if (rep_SYMBOLP(handler))
     {
-	if (rep_fh_env == Qt)
+	repv fh_env = get_fh_env ();
+	if (fh_env == Qt)
+	{
+	    rep_USE_DEFAULT_ENV;
 	    handler = Fsymbol_value (handler, Qt);
+	}
 	else
 	{
-	    repv tem = Fassq (handler, rep_fh_env);
+	    repv tem = Fassq (handler, fh_env);
 	    if (tem && rep_CONSP(tem))
 	    {
 		if (rep_CDR(tem) == Qt)
@@ -268,13 +281,18 @@ rep_call_file_handler(repv handler, int op, repv sym, int nargs, ...)
 	}
     }
 
-    rep_push_regexp_data(&matches);
-    op_data.next = rep_blocked_ops[op];
-    rep_blocked_ops[op] = &op_data;
-    /* handler and arg_list are automatically gc-protected by rep_funcall */
-    res = rep_funcall(handler, arg_list, rep_FALSE);
-    rep_blocked_ops[op] = op_data.next;
-    rep_pop_regexp_data();
+    if (handler != rep_NULL && !rep_VOIDP (handler))
+    {
+	rep_push_regexp_data(&matches);
+	op_data.next = rep_blocked_ops[op];
+	rep_blocked_ops[op] = &op_data;
+	/* handler and arg_list are automatically protected by rep_funcall */
+	res = rep_funcall(handler, arg_list, rep_FALSE);
+	rep_blocked_ops[op] = op_data.next;
+	rep_pop_regexp_data();
+    }
+    else
+	res = rep_NULL;
 
     rep_POP_CALL(lc);
     return res;
@@ -1540,17 +1558,12 @@ Returns the name of a unique file in the local filing system.
 }
 
 DEFUN("set-file-handler-environment", Fset_file_handler_environment,
-      Sset_file_handler_environment, (repv env), rep_Subr1) /*
+      Sset_file_handler_environment, (repv env, repv structure), rep_Subr2) /*
 ::doc:set-file-handler-environment::
 set-file-handler-environment ENV
 ::end:: */
 {
-    if (rep_call_stack != 0)
-    {
-	if (rep_LISTP (env))
-	    rep_call_stack->saved_fh_env = env;
-    }
-    return Qt;
+    return F_structure_set (structure, Qfh_env_key, env);
 }
 
 
@@ -1561,6 +1574,9 @@ rep_files_init(void)
 {
     repv tem;
 
+    Qfh_env_key = Fmake_symbol (rep_VAL (&str_fh_env_key));
+    rep_mark_static (&Qfh_env_key);
+
     rep_INTERN_SPECIAL(file_handler_alist);
     Fset (Qfile_handler_alist, Qnil);
 
