@@ -1,6 +1,6 @@
 /* gettext.c -- wrap some i18n functions when available
    Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
-   $Id: gettext.c,v 1.6 2000/02/28 08:29:28 john Exp $
+   $Id: gettext.c,v 1.8 2000/06/23 15:24:20 john Exp $
 
    This file is part of librep.
 
@@ -33,9 +33,7 @@ extern char *gnu_gettext (const char *msgid);
 extern char *gnu_textdomain (const char *domainname);
 extern char *gnu_bindtextdomain (const char *domainname, const char *dirname);
 
-DEFSYM(gettext, "gettext");
-
-DEFUN("_", Fgettext, Sgettext, (repv in), rep_Subr1)
+DEFUN("gettext", Fgettext, Sgettext, (repv in), rep_Subr1)
 {
     char *out;
     rep_DECLARE1(in, rep_STRINGP);
@@ -76,13 +74,22 @@ DEFUN("textdomain", Ftextdomain, Stextdomain, (repv dom), rep_Subr1)
 
 /* DL hooks */
 
+DEFSTRING(underscore, "_");
+
 repv
 rep_dl_init(void)
 {
+    repv tem = rep_push_structure ("gettext"), ret;
     rep_ADD_SUBR(Sgettext);
     rep_ADD_SUBR(Sbindtextdomain);
     rep_ADD_SUBR(Stextdomain);
+    ret = rep_pop_structure (tem);
 
-    rep_INTERN(gettext);
-    return Qgettext;
+    /* Update binding of `_' in `rep' structure to point at the
+       gettext function */
+    tem = rep_push_structure ("rep");
+    Fset (Fintern (rep_VAL (&underscore), Qnil), rep_VAL (&Sgettext));
+    rep_pop_structure (tem);
+
+    return ret;
 }
