@@ -1,6 +1,6 @@
 /* lispcmds.c -- Lots of standard Lisp functions
    Copyright (C) 1993, 1994 John Harper <john@dcs.warwick.ac.uk>
-   $Id: lispcmds.c,v 1.73 2000/03/28 23:51:50 john Exp $
+   $Id: lispcmds.c,v 1.96 2000/07/14 15:15:36 john Exp $
 
    This file is part of Jade.
 
@@ -21,10 +21,12 @@
 #define _GNU_SOURCE
 
 #include "repint.h"
+#include "build.h"
 
 #include <string.h>
 #include <stdlib.h>
 #include <ctype.h>
+#include <assert.h>
 
 #ifdef NEED_MEMORY_H
 # include <memory.h>
@@ -39,13 +41,13 @@ DEFSYM(load_path, "load-path");
 DEFSYM(dl_load_path, "dl-load-path");
 DEFSYM(after_load_alist, "after-load-alist");
 DEFSYM(provide, "provide");
-DEFSYM(features, "features");
 DEFSYM(rep_directory, "rep-directory");
 DEFSYM(lisp_lib_directory, "lisp-lib-directory");
 DEFSYM(site_lisp_directory, "site-lisp-directory");
 DEFSYM(exec_directory, "exec-directory");
 DEFSYM(documentation_file, "documentation-file");
 DEFSYM(documentation_files, "documentation-files");
+DEFSYM(_load_suffixes, "%load-suffixes");
 DEFSYM(dl_load_reloc_now, "dl-load-reloc-now");
 DEFSYM(load_filename, "load-filename"); /*
 ::doc:load-path::
@@ -88,7 +90,7 @@ While using the `load' function to load a Lisp library, this variable is
 set to the name of the file being loaded.
 ::end:: */
 
-DEFUN("quote", Fquote, Squote, (repv args), rep_SF) /*
+DEFUN("quote", Fquote, Squote, (repv args, repv tail_posn), rep_SF) /*
 ::doc:quote::
 quote ARG
 'ARG
@@ -101,7 +103,7 @@ Returns ARG.
     return rep_signal_missing_arg(1);
 }
 
-DEFUN("lambda", Flambda, Slambda, (repv args), rep_SF) /*
+DEFUN("lambda", Flambda, Slambda, (repv args, repv tail_posn), rep_SF) /*
 ::doc:lambda::
 lambda LAMBDA-LIST BODY...
 
@@ -220,7 +222,7 @@ new list which is returned.
     repv *resend = &res;
     while(rep_CONSP(args))
     {
-	if(!rep_LISTP(rep_CAR(args)))
+	if(!rep_LISTP(rep_CAR(args)) && rep_CDR (args) != Qnil)
 	    return rep_signal_arg_error(rep_CAR(args), i);
 	if(rep_CONSP(rep_CAR(args)) && rep_CONSP(rep_CDR(args)))
 	{
@@ -258,7 +260,7 @@ to the beginning of the next list. Returns the new list.
     while(rep_CONSP(args))
     {
 	repv tmp = rep_CAR(args);
-	if(!rep_LISTP(tmp))
+	if(!rep_LISTP(tmp) && rep_CDR (args) != Qnil)
 	    return rep_signal_arg_error(tmp, i);
 	if(rep_CONSP(tmp))
 	{
@@ -274,6 +276,8 @@ to the beginning of the next list. Returns the new list.
 	    }
 	    resend = &rep_CDR(tmp);
 	}
+	else
+	    *resend = tmp;
 	args = rep_CDR(args);
     }
     return(res);
@@ -283,30 +287,30 @@ DEFUN("rplaca", Frplaca, Srplaca, (repv cons, repv car), rep_Subr2) /*
 ::doc:rplaca::
 rplaca CONS-CELL NEW-CAR
 
-Sets the value of the car slot in CONS-CELL to NEW-CAR. Returns the new
-value.
+Sets the value of the car slot in CONS-CELL to NEW-CAR.
+Returns the CONS-CELL.
 ::end:: */
 {
     rep_DECLARE1(cons, rep_CONSP);
     if(!rep_CONS_WRITABLE_P(cons))
 	return Fsignal(Qsetting_constant, rep_LIST_1(cons));
     rep_CAR(cons) = car;
-    return(car);
+    return(cons);
 }
 
 DEFUN("rplacd", Frplacd, Srplacd, (repv cons, repv cdr), rep_Subr2) /*
 ::doc:rplacd::
 rplacd CONS-CELL NEW-CDR
 
-Sets the value of the cdr slot in CONS-CELL to NEW-CAR. Returns the new
-value.
+Sets the value of the cdr slot in CONS-CELL to NEW-CDR.
+Returns the CONS-CELL.
 ::end:: */
 {
     rep_DECLARE1(cons, rep_CONSP);
     if(!rep_CONS_WRITABLE_P(cons))
 	return Fsignal(Qsetting_constant, rep_LIST_1(cons));
     rep_CDR(cons) = cdr;
-    return(cdr);
+    return(cons);
 }
 
 DEFUN("reverse", Freverse, Sreverse, (repv head), rep_Subr1) /*
@@ -668,6 +672,33 @@ from the matched ELT, ie,
     return(Qnil);
 }
 
+DEFUN("memql", Fmemql, Smemql, (repv elt, repv list), rep_Subr2) /*
+::doc:memql::
+memql ELT LIST
+
+If ELT is a member of list LIST then return the tail of the list starting
+from the matched ELT. `memql' uses `eql' to compare list items.
+::end:: */
+{
+    rep_DECLARE2 (list, rep_LISTP);
+    while (rep_CONSP (list))
+    {
+	if (elt == rep_CAR (list))
+	    return list;
+	else
+	{
+	    repv tem = Feql (elt, rep_CAR (list));
+	    if (tem && tem != Qnil)
+		return list;
+	}
+	list = rep_CDR (list);
+	rep_TEST_INT;
+	if (rep_INTERRUPTP)
+	    return rep_NULL;
+    }
+    return Qnil;
+}
+
 DEFUN("delete", Fdelete, Sdelete, (repv elt, repv list), rep_Subr2) /*
 ::doc:delete::
 delete ELT LIST
@@ -900,8 +931,8 @@ can be a vector or a string. INDEX starts at zero.
 	    return(rep_VECTI(array, rep_INT(index)));
     }
     else
-	return(Fsignal(Qbad_arg, rep_list_2(array, rep_MAKE_INT(1))));
-    return(rep_signal_arg_error(index, 2));
+	return rep_signal_arg_error (array, 1);
+    return rep_signal_arg_error (index, 2);
 }
 
 DEFUN("make-string", Fmake_string, Smake_string, (repv len, repv init), rep_Subr2) /*
@@ -1085,8 +1116,7 @@ Returns the number of elements in SEQUENCE (a string, list or vector).
 	    return(rep_MAKE_INT(0));
 	/* FALL THROUGH */
     default:
-	Fsignal(Qbad_arg, rep_list_2(sequence, rep_MAKE_INT(1)));
-	return(rep_NULL);
+	return rep_signal_arg_error (sequence, 1);
     }
 }
 
@@ -1151,63 +1181,7 @@ Return the element of SEQUENCE at position INDEX (counting from zero).
 	return(Faref(seq, index));
 }
 
-DEFUN("prog1", Fprog1, Sprog1, (repv args), rep_SF) /*
-::doc:prog1::
-prog1 FORM1 FORMS...
-
-First evals FORM1 then FORMS, returns the value that FORM1 gave.
-::end:: */
-{
-    if(rep_CONSP(args))
-    {
-	repv res;
-	rep_GC_root gc_args, gc_res;
-	rep_PUSHGC(gc_args, args);
-	res = Feval(rep_CAR(args));
-	if(res)
-	{
-	    rep_PUSHGC(gc_res, res);
-	    if(!Fprogn(rep_CDR(args)))
-		res = rep_NULL;
-	    rep_POPGC;
-	}
-	rep_POPGC;
-	return(res);
-    }
-    return rep_signal_missing_arg(1);
-}
-
-DEFUN("while", Fwhile, Swhile, (repv args), rep_SF) /*
-::doc:while::
-while CONDITION FORMS...
-
-Eval CONDITION, if it is non-nil then execute FORMS and repeat the
-procedure, else return nil.
-::end:: */
-{
-    if(rep_CONSP(args))
-    {
-	rep_GC_root gc_args;
-	repv cond = rep_CAR(args), wval, body = rep_CDR(args);
-	rep_PUSHGC(gc_args, args);
-	while((wval = Feval(cond)) && !rep_NILP(wval))
-	{
-	    rep_TEST_INT;
-	    if(rep_INTERRUPTP || !Fprogn(body))
-	    {
-		wval = rep_NULL;
-		break;
-	    }
-	}
-	rep_POPGC;
-	if(!wval)
-	    return(rep_NULL);
-	return(Qnil);
-    }
-    return rep_signal_missing_arg(1);
-}
-
-DEFUN("cond", Fcond, Scond, (repv args), rep_SF) /*
+DEFUN("cond", Fcond, Scond, (repv args, repv tail_posn), rep_SF) /*
 ::doc:cond::
 cond (CONDITION FORMS... ) ...
 
@@ -1234,13 +1208,13 @@ like the last else in an else-if statement in C.
     while(rep_CONSP(args) && rep_CONSP(rep_CAR(args)))
     {
 	repv cndlist = rep_CAR(args);
-	if(!(res = Feval(rep_CAR(cndlist))))
+	if(!(res = rep_eval(rep_CAR(cndlist), Qnil)))
 	    break;
 	if(!rep_NILP(res))
 	{
 	    if(rep_CONSP(rep_CDR(cndlist)))
 	    {
-		if(!(res = Fprogn(rep_CDR(cndlist))))
+		if(!(res = Fprogn(rep_CDR(cndlist), tail_posn)))
 		    break;
 	    }
 	    break;
@@ -1251,7 +1225,7 @@ like the last else in an else-if statement in C.
     return(res);
 }
 
-DEFUN("case", Fcase, Scase, (repv args), rep_SF) /*
+DEFUN("case", Fcase, Scase, (repv args, repv tail_posn), rep_SF) /*
 ::doc:case::
 case KEY CLAUSES...
 
@@ -1273,7 +1247,7 @@ undefined.
 	return rep_signal_missing_arg (0);
     
     rep_PUSHGC(gc_args, args);
-    key = Feval (rep_CAR(args));
+    key = rep_eval (rep_CAR(args), Qnil);
     rep_POPGC;
     if (key == rep_NULL)
 	return rep_NULL;
@@ -1288,8 +1262,8 @@ undefined.
 	    repv ptr = rep_CAAR(args);
 	    while (rep_CONSP(ptr))
 	    {
-		if (rep_CAR(ptr) == key)
-		    return Fprogn (rep_CDAR(args));
+		if (Feql (key, rep_CAR(ptr)) != Qnil)
+		    return Fprogn (rep_CDAR(args), tail_posn);
 		ptr = rep_CDR(ptr);
 		rep_TEST_INT;
 		if (rep_INTERRUPTP)
@@ -1297,7 +1271,7 @@ undefined.
 	    }
 	}
 	else if (rep_CAAR(args) == Qt)
-	    return Fprogn (rep_CDAR(args));
+	    return Fprogn (rep_CDAR(args), tail_posn);
 	else
 	    return rep_signal_arg_error (rep_CAR(args), i);
 	args = rep_CDR(args);
@@ -1333,7 +1307,7 @@ ie,
 	if(!rep_NILP(Flistp(rep_CAR(args))))
 	    *last = rep_CAR(args);
 	else
-	    return(Fsignal(Qbad_arg, rep_LIST_1(rep_CAR(args))));
+	    return rep_signal_arg_error (rep_CAR (args), -1);
 	return(Ffuncall(list));
     }
     return rep_signal_missing_arg(1);
@@ -1352,9 +1326,9 @@ load_file_exists_p (repv name)
     return tem;
 }
 
-DEFUN_INT("load", Fload, Sload, (repv file, repv noerr_p, repv nopath_p, repv nosuf_p, repv in_env), rep_Subr5, "fLisp file to load:") /*
+DEFUN_INT("load", Fload, Sload, (repv file, repv noerr_p, repv nopath_p, repv nosuf_p, repv unused), rep_Subr5, "fLisp file to load:") /*
 ::doc:load::
-load FILE [NO-ERROR] [NO-PATH] [NO-SUFFIX] [IN-CURRENT-ENVIRONMENT]
+load FILE [NO-ERROR] [NO-PATH] [NO-SUFFIX]
 
 Attempt to open and then read-and-eval the file of Lisp code FILE.
 
@@ -1366,10 +1340,6 @@ If NO-ERROR is non-nil no error is signalled if FILE can't be found. If
 NO-PATH is non-nil the `load-path' variable is not used, just the value
 of FILE. If NO-SUFFIX is non-nil no suffixes are appended to FILE.
 
-If IN-CURRENT-ENVIRONMENT is non-nil then the forms contained by FILE
-are evaluated within the current lexical environment; by default a new
-environment is created for each file loaded.
-
 If the compiled version is older than it's source code, the source code is
 loaded and a warning is displayed.
 ::end:: */
@@ -1377,17 +1347,18 @@ loaded and a warning is displayed.
     /* Avoid the need to protect these args from GC. */
     rep_bool no_error_p = !rep_NILP(noerr_p);
     rep_bool no_suffix_p = !rep_NILP(nosuf_p);
-    rep_bool in_current_env = !rep_NILP(in_env);
     rep_bool interp_mode = Fsymbol_value (Qinterpreted_mode, Qt) != Qnil;
 
     repv name = Qnil, path;
     repv dir = rep_NULL, try = rep_NULL;
+    repv result = rep_NULL;
+    repv suffixes;
 
 #ifdef HAVE_DYNAMIC_LOADING
     rep_bool trying_dl = rep_FALSE;
 #endif
 
-    rep_GC_root gc_file, gc_name, gc_path, gc_dir, gc_try;
+    rep_GC_root gc_file, gc_name, gc_path, gc_dir, gc_try, gc_result, gc_suffixes;
 
     rep_DECLARE1(file, rep_STRINGP);
     if(rep_NILP(nopath_p))
@@ -1399,11 +1370,16 @@ loaded and a warning is displayed.
     else
 	path = Fcons(rep_null_string(), Qnil);
 
+    suffixes = F_structure_ref (rep_structure, Q_load_suffixes);
+    if (!suffixes || !rep_CONSP (suffixes))
+	no_suffix_p = rep_TRUE;
+
     rep_PUSHGC(gc_name, name);
     rep_PUSHGC(gc_file, file);
     rep_PUSHGC(gc_path, path);
     rep_PUSHGC(gc_dir, dir);
     rep_PUSHGC(gc_try, try);
+    rep_PUSHGC(gc_suffixes, suffixes);
 
     /* Scan the path for the file to load. */
 research:
@@ -1415,10 +1391,9 @@ research:
 	    if(dir == rep_NULL || !rep_STRINGP(dir))
 		goto path_error;
 
-	    if(!no_suffix_p)
+	    if(trying_dl || !no_suffix_p)
 	    {
 		repv tem;
-		static char *suffixes[3] = { ".jl", ".jlc" };
 		int i = 1;
 		if (!trying_dl && interp_mode)
 		    i = 0;
@@ -1438,31 +1413,36 @@ research:
 		    }
 		    else
 #endif
-			try = rep_concat2(rep_STR(dir), suffixes[i]);
-
-		    if (try == rep_NULL || !rep_STRINGP (try))
-			goto path_error;
-
-		    tem = load_file_exists_p (try);
-		    if(!tem)
-			goto path_error;
-		    if(tem != Qnil)
 		    {
-			if(name != Qnil)
+			repv sfx = ((i == 0)
+				    ? rep_CAR(suffixes) : rep_CDR(suffixes));
+			if (rep_STRINGP (sfx))
+			    try = rep_concat2(rep_STR(dir), rep_STR(sfx));
+		    }
+
+		    if (try && rep_STRINGP (try))
+		    {
+			tem = load_file_exists_p (try);
+			if(!tem)
+			    goto path_error;
+			if(tem != Qnil)
 			{
-			    if(rep_file_newer_than(try, name))
+			    if(name != Qnil)
 			    {
-				if (rep_message_fun != 0)
-				    (*rep_message_fun)(rep_messagef,"Warning: %s newer than %s, using %s", rep_STR(try), rep_STR(name), rep_STR(try));
-				name = try;
+				if(rep_file_newer_than(try, name))
+				{
+				    if (rep_message_fun != 0)
+					(*rep_message_fun)(rep_messagef, "Warning: %s newer than %s, using %s", rep_STR(try), rep_STR(name), rep_STR(try));
+				    name = try;
+				}
 			    }
+			    else
+				name = try;
 			}
-			else
-			    name = try;
 		    }
 		}
 	    }
-	    if(name == Qnil)
+	    if(!trying_dl && name == Qnil && no_suffix_p)
 	    {
 		/* Try without a suffix */
 		repv tem = load_file_exists_p (dir);
@@ -1495,7 +1475,7 @@ research:
 #endif
 
 path_error:
-    rep_POPGC; rep_POPGC; rep_POPGC; rep_POPGC; rep_POPGC;
+    rep_POPGC; rep_POPGC; rep_POPGC; rep_POPGC; rep_POPGC; rep_POPGC;
 
     if(rep_NILP(name))
     {
@@ -1508,11 +1488,10 @@ path_error:
 #ifdef HAVE_DYNAMIC_LOADING
     if(trying_dl)
     {
-	void *handle;
 	rep_PUSHGC(gc_file, file);
-	handle = rep_open_dl_library(name);
+	result = rep_open_dl_library (name);
 	rep_POPGC;
-	if (handle == 0)
+	if (result == rep_NULL)
 	    return rep_NULL;
     }
     else
@@ -1526,7 +1505,9 @@ path_error:
 	int c;
 
 	rep_PUSHGC(gc_file, file);
+	rep_PUSHGC(gc_stream, name);
 	stream = Fopen_file(name, Qread);
+	rep_POPGC;
 	if(!stream || !rep_FILEP(stream))
 	{
 	    rep_POPGC;
@@ -1542,18 +1523,14 @@ path_error:
 	lc.args = Qnil;
 	lc.args_evalled_p = Qnil;
 	rep_PUSH_CALL(lc);
+	rep_env = Qnil;
 
-	if (!in_current_env)
-	{
-	    rep_env = Qt;
-	    rep_special_env = Fcons (Qnil, Qt);
-	}
-
+	result = Qnil;
 	c = rep_stream_getc(stream);
 	while((c != EOF) && (tem = rep_readl(stream, &c)))
 	{
 	    rep_TEST_INT;
-	    if(rep_INTERRUPTP || !Feval(tem))
+	    if(rep_INTERRUPTP || !(result = rep_eval(tem, Qnil)))
 	    {
 		rep_unbind_symbols (bindings);
 		rep_POP_CALL(lc);
@@ -1571,13 +1548,15 @@ path_error:
 	}
 	rep_POP_CALL(lc);
 	rep_POPGC; rep_POPGC;
+	rep_PUSHGC (gc_result, result);
 	rep_unbind_symbols (bindings);
 	Fclose_file (stream);
-	rep_POPGC;
+	rep_POPGC; rep_POPGC;
     }
 
     /* Loading succeeded. Look for an applicable item in
        the after-load-alist. */
+    rep_PUSHGC (gc_result, result);
     rep_PUSHGC (gc_file, file);
     {
 	repv tem;
@@ -1592,8 +1571,16 @@ again:
 		Fset(Qafter_load_alist,
 		     Fdelq(tem, Fsymbol_value (Qafter_load_alist, Qt)));
 	    
-		/* Then evaluate it */
-		Fprogn(rep_CDR(tem));
+		/* Then call it */
+		tem = rep_CDR (tem);
+		while (rep_CONSP (tem) && !rep_INTERRUPTP)
+		{
+		    rep_GC_root gc_tem;
+		    rep_PUSHGC (gc_tem, tem);
+		    rep_call_lisp0 (rep_CAR (tem));
+		    rep_POPGC;
+		    tem = rep_CDR (tem);
+		}
 
 		/* Try for another entry */
 		goto again;
@@ -1601,8 +1588,9 @@ again:
 	}
     }
     rep_POPGC;
+    rep_POPGC;
 
-    return Qt;
+    return result;
 }
 
 DEFUN("equal", Fequal, Sequal, (repv val1, repv val2), rep_Subr2) /*
@@ -1718,9 +1706,15 @@ Returns t if STRING1 is `less' than STRING2, ignoring case.
 	args = rep_CDR(args);				\
 	while(rep_CONSP(args))				\
 	{						\
-	    if(!(rep_value_cmp(pred, rep_CAR(args)) op 0)) \
+	    int sign;					\
+	    repv tem = rep_CAR (args);			\
+	    if (rep_NUMBERP (pred) || rep_NUMBERP (tem)) \
+		sign = rep_compare_numbers (pred, tem);	\
+	    else					\
+		sign = rep_value_cmp(pred, tem);	\
+	    if(!(sign op 0)) 				\
 		return Qnil;				\
-	    pred = rep_CAR(args);			\
+	    pred = tem;					\
 	    args = rep_CDR(args);			\
 	    i++;					\
 	    rep_TEST_INT;				\
@@ -1736,7 +1730,8 @@ DEFUN("=", Fnum_eq, Snum_eq, (repv args), rep_SubrN) /*
 = ARG1 ARG2 [ARG3 ...]
 
 Returns t if each value is the same as every other value. (Using
-`equal' to compare values.)
+`equal' to compare values, except for numbers, where exactness is
+ignored.)
 ::end:: */
 {
     APPLY_COMPARISON(==)
@@ -1747,7 +1742,8 @@ DEFUN("/=", Fnum_noteq, Snum_noteq, (repv args), rep_SubrN) /*
 /= ARG1 ARG2 ...
 
 Returns t if each value is different from every other value. (Using
-`equal' to compare values.)
+`equal' to compare values, except for numbers, where exactness is
+ignored.)
 ::end:: */
 {
     repv ret = Fnum_eq (args);
@@ -1802,56 +1798,6 @@ limited to numbers, it can do strings, positions, marks, etc as well.
     APPLY_COMPARISON(<=)
 }
 
-DEFUN("max", Fmax, Smax, (repv args), rep_SubrN) /*
-::doc:max::
-max ARGS...
-
-Returns the greatest of its arguments. There must be at least two
-arguments.
-::end:: */
-{
-    repv max;
-    if(!rep_CONSP(args) || !rep_CONSP(rep_CDR(args)))
-	return rep_signal_missing_arg(rep_CONSP(args) ? 2 : 1);
-    max = rep_CAR(args);
-    args = rep_CDR(args);
-    while(rep_CONSP(args))
-    {
-	repv tem = rep_CAR(args);
-	args = rep_CDR(args);
-	max = (rep_value_cmp(max, tem) >= 0) ? max : tem;
-	rep_TEST_INT;
-	if(rep_INTERRUPTP)
-	    return rep_NULL;
-    }
-    return max;
-}
-
-DEFUN("min", Fmin, Smin, (repv args), rep_SubrN) /*
-::doc:min::
-min ARGS...
-
-Returns the smallest of its arguments. There must be at least two
-arguments.
-::end:: */
-{
-    repv min;
-    if(!rep_CONSP(args) || !rep_CONSP(rep_CDR(args)))
-	return rep_signal_missing_arg(rep_CONSP(args) ? 2 : 1);
-    min = rep_CAR(args);
-    args = rep_CDR(args);
-    while(rep_CONSP(args))
-    {
-	repv tem = rep_CAR(args);
-	args = rep_CDR(args);
-	min = (rep_value_cmp(min, tem) <= 0) ? min : tem;
-	rep_TEST_INT;
-	if(rep_INTERRUPTP)
-	    return rep_NULL;
-    }
-    return min;
-}
-
 DEFUN("null", Fnull, Snull, (repv arg), rep_Subr1) /*
 ::doc:null::
 null ARG
@@ -2082,204 +2028,67 @@ returned.
     return res;
 }
 
-DEFUN("catch", Fcatch, Scatch, (repv args), rep_SF) /*
-::doc:catch::
-catch TAG FORMS...
+DEFUN("call-with-exception-handler", Fcall_with_exception_handler,
+      Scall_with_exception_handler, (repv thunk, repv handler_thunk),
+      rep_Subr2) /*
+::doc:call-with-exception-handler::
+call-with-exception-handler THUNK HANDLER-THUNK
 
-Evaluates FORMS, non-local exits are allowed with `(throw TAG)'.
-The value of `catch' is either the value of the last FORM or the
-value given to the throw command.
-
-There are several pre-defined `catch'es which are,
-  'defun
-     Around all defuns, the `return' command uses this, it basically does
-     (throw 'defun X).
-  'exit
-     Exits one level of recursive-editing (but doesn't work in the top
-     level.
-  'top-level
-     At the top-level recursive-edit (ie, the one which you're in when
-     the editor is started).
-  'quit
-     Kills the editor.
+Call THUNK and return its value. However if an exception of any form
+occurs, call HANDLER-THUNK with a single argument, the exception data,
+and return its value.
 ::end:: */
     /* Non-local exits don't bother with jmp_buf's and the like, they just
        unwind normally through all levels of recursion with a rep_NULL result.
        This is slow but it's easy to work with.  */
 {
-    if(rep_CONSP(args))
+    rep_GC_root gc_handler;
+    repv ret;
+
+    rep_DECLARE (1, thunk, Ffunctionp (thunk) != Qnil);
+    rep_DECLARE (2, handler_thunk, Ffunctionp (handler_thunk) != Qnil);
+
+    rep_PUSHGC (gc_handler, handler_thunk);
+    ret = rep_call_lisp0 (thunk);
+    rep_POPGC;
+    if (ret == rep_NULL)
     {
-	repv tag, res = rep_NULL;
-	rep_GC_root gc_args, gc_tag;
-	rep_PUSHGC(gc_args, args);
-	tag = Feval(rep_CAR(args));
-	if(tag)
-	{
-	    rep_PUSHGC(gc_tag, tag);
-	    if(!(res = Fprogn(rep_CDR(args))))
-	    {
-		if(rep_throw_value && (rep_CAR(rep_throw_value) == tag))
-		{
-		    res = rep_CDR(rep_throw_value);
-		    rep_throw_value = rep_NULL;
-		}
-	    }
-	    rep_POPGC;
-	}
-	rep_POPGC;
-	return(res);
+	repv data = rep_throw_value;
+	rep_throw_value = rep_NULL;
+	assert (data != rep_NULL);
+	ret = rep_call_lisp1 (handler_thunk, data);
     }
-    return rep_signal_missing_arg(1);
+    return ret;
 }
 
-DEFUN("throw", Fthrow, Sthrow, (repv tag, repv val), rep_Subr2) /*
-::doc:throw::
-throw TAG repv
+DEFUN("raise-exception", Fraise_exception,
+      Sraise_exception, (repv ex), rep_Subr1) /*
+::doc:raise-exception::
+raise-exception DATA
 
-Performs a non-local exit to the `catch' waiting for TAG and return
-repv from it. TAG and repv are both evaluated fully.
+Raise the exception represented by the cons cell DATA.
 ::end:: */
 {
     /* Only one thing can use `rep_throw_value' at once.  */
-    if(!rep_throw_value)
-	rep_throw_value = Fcons(tag, val);
-    return(rep_NULL);
+    rep_DECLARE1 (ex, rep_CONSP);
+    if (rep_throw_value == rep_NULL)
+	rep_throw_value = ex;
+    return rep_NULL;
 }
 
-DEFUN("unwind-protect", Funwind_protect, Sunwind_protect, (repv args), rep_SF) /*
-::doc:unwind-protect::
-unwind-protect BODY CLEANUP-FORMS...
-
-Return the result of evaluating BODY. When execution leaves the dynamic
-extent of BODY evaluate `(progn CLEANUP-FORMS)' (even if exiting due to
-an exception within BODY).
-
-Note that when BODY is exited by calling a continuation, it is
-undefined whether or not CLEANUP-FORMS will be evaluated.
-::end:: */
-{
-    if(rep_CONSP(args))
-    {
-	repv res, throwval;
-	rep_GC_root gc_args, gc_res, gc_throwval;
-	rep_PUSHGC(gc_args, args);
-	res = Feval(rep_CAR(args));
-	rep_PUSHGC(gc_res, res);
-	throwval = rep_throw_value;
-	rep_throw_value = rep_NULL;
-	rep_PUSHGC(gc_throwval, throwval);
-	if(!Fprogn(rep_CDR(args)))
-	    res = rep_NULL;
-	/* Only reinstall the old throw if it's actually an error and there
-	   was no error in the cleanup forms. This way the newest error
-	   takes precedence. */
-	if(throwval != 0 && rep_throw_value == 0)
-	    rep_throw_value = throwval;
-	rep_POPGC; rep_POPGC; rep_POPGC;
-	return(res);
-    }
-    return rep_signal_missing_arg(1);
+/* XXX compatibility */
+repv Fthrow (repv tag, repv value) {
+    return Fraise_exception (Fcons (tag, value));
 }
 
-DEFUN("with-object", Fwith_object, Swith_object, (repv args), rep_SF) /*
-::doc:with-object::
-with-object ARG FORMS...
-
-Evaluate ARG and make its value ``current'' in some way meaningful for
-the data type, evaluate all FORMS, then return to the old current value
-of whatever was changed. Return the value of the last FORM evaluated.
-::end:: */
-{
-    repv res;
-    if(rep_CONSP(args))
-    {
-	rep_GC_root gc_args;
-	rep_PUSHGC(gc_args, args);
-	res = Feval(rep_CAR(args));
-	if (res != rep_NULL)
-	{
-	    repv handle = rep_bind_object(res);
-	    if (handle != rep_NULL)
-	    {
-		rep_GC_root gc_handle;
-		rep_PUSHGC(gc_handle, handle);
-		res = Fprogn(rep_CDR(args));
-		rep_POPGC;
-		rep_PUSHGC(gc_handle, res);
-		rep_unbind_object(handle);
-		rep_POPGC;
-	    }
-	    else
-		res = rep_NULL;
-	}
-	rep_POPGC;
-    }
-    else
-	res = rep_signal_arg_error(res, 1);
-    return res;
-}
-
-DEFUN("featurep", Ffeaturep, Sfeaturep, (repv feature), rep_Subr1) /*
-::doc:featurep::
-featurep FEATURE
-
-Return non-nil if feature FEATURE has already been loaded.
-::end:: */
-{
-    repv value;
-    rep_DECLARE1 (feature, rep_SYMBOLP);
-    value = Fsymbol_value (Qfeatures, Qnil);
-    if (value != rep_NULL)
-	return Fmemq (feature, value);
-    else
-	return rep_NULL;
-}
-
-DEFUN("provide", Fprovide, Sprovide, (repv feature), rep_Subr1) /*
-::doc:provide::
-provide FEATURE
-
-Show that the feature FEATURE (a symbol) has been loaded.
-::end:: */
-{
-    repv value;
-    rep_DECLARE1 (feature, rep_SYMBOLP);
-    value = Fsymbol_value (Qfeatures, Qnil);
-    if (value != rep_NULL)
-    {
-	repv tem = Fmemq (feature, value);
-	if (tem && tem == Qnil)
-	    Fset (Qfeatures, Fcons (feature, value));
-	return feature;
-    }
-    else
-	return rep_NULL;
-}
-
-DEFUN_INT("require", Frequire, Srequire, (repv feature, repv file), rep_Subr2,
-	  "SFeature to load:") /*
-::doc:require::
-require FEATURE [FILE]
-
-If FEATURE (a symbol) has not already been loaded, load it. The file
-loaded is either FILE (if given), or the print name of FEATURE.
-::end:: */
-{
-    repv tem;
-    rep_DECLARE1 (feature, rep_SYMBOLP);
-    tem = Ffeaturep (feature);
-    if (tem && tem == Qnil)
-    {
-	if (!rep_STRINGP(file))
-	    file = rep_SYM(feature)->name;
-	return Fload (file, Qnil, Qnil, Qnil, Qnil);
-    }
-    return feature;
-}
+DEFSTRING(jl, ".jl");
+DEFSTRING(jlc, ".jlc");
 
 void
 rep_lispcmds_init(void)
 {
+    DEFSTRING (common_exec, REP_COMMON_EXEC_DIRECTORY);
+
     rep_ADD_SUBR(Squote);
     rep_ADD_SUBR(Slambda);
     rep_ADD_SUBR(Scar);
@@ -2305,6 +2114,7 @@ rep_lispcmds_init(void)
     rep_ADD_SUBR(Sfilter);
     rep_ADD_SUBR(Smember);
     rep_ADD_SUBR(Smemq);
+    rep_ADD_SUBR(Smemql);
     rep_ADD_SUBR(Sdelete);
     rep_ADD_SUBR(Sdelq);
     rep_ADD_SUBR(Sdelete_if);
@@ -2320,8 +2130,6 @@ rep_lispcmds_init(void)
     rep_ADD_SUBR(Slength);
     rep_ADD_SUBR(Scopy_sequence);
     rep_ADD_SUBR(Selt);
-    rep_ADD_SUBR(Sprog1);
-    rep_ADD_SUBR(Swhile);
     rep_ADD_SUBR(Scond);
     rep_ADD_SUBR(Scase);
     rep_ADD_SUBR(Sapply);
@@ -2338,8 +2146,6 @@ rep_lispcmds_init(void)
     rep_ADD_SUBR(Sgethan);
     rep_ADD_SUBR(Sltthan);
     rep_ADD_SUBR(Slethan);
-    rep_ADD_SUBR(Smax);
-    rep_ADD_SUBR(Smin);
     rep_ADD_SUBR(Snull);
     rep_ADD_SUBR(Satom);
     rep_ADD_SUBR(Sconsp);
@@ -2354,18 +2160,10 @@ rep_lispcmds_init(void)
     rep_ADD_SUBR(Ssequencep);
     rep_ADD_SUBR(Ssubr_name);
     rep_ADD_SUBR(Scall_hook);
-    rep_ADD_SUBR(Scatch);
-    rep_ADD_SUBR(Sthrow);
-    rep_ADD_SUBR(Sunwind_protect);
-    rep_ADD_SUBR(Swith_object);
-    rep_ADD_SUBR(Sfeaturep);
-    rep_ADD_SUBR(Sprovide);
-    rep_ADD_SUBR_INT(Srequire);
+    rep_ADD_SUBR(Scall_with_exception_handler);
+    rep_ADD_SUBR(Sraise_exception);
 
     rep_INTERN(provide);
-    rep_INTERN_SPECIAL(features);
-    if (Fboundp (Qfeatures) == Qnil)
-	Fset (Qfeatures, Qnil);
 
     rep_INTERN_SPECIAL(rep_directory);
     if(getenv("REPDIR") != 0)
@@ -2397,7 +2195,10 @@ rep_lispcmds_init(void)
     if(getenv("REPDOCFILE") != 0)
 	Fset (Qdocumentation_file, rep_string_dup(getenv("REPDOCFILE")));
     else
-	Fset (Qdocumentation_file, rep_string_dup(REP_DOC_FILE));
+    {
+	DEFSTRING (doc_file, REP_DOC_FILE);
+	Fset (Qdocumentation_file, rep_VAL (&doc_file));
+    }
 
     rep_INTERN_SPECIAL(documentation_files);
     Fset (Qdocumentation_files,
@@ -2423,7 +2224,7 @@ rep_lispcmds_init(void)
     }
     Fset (Qdl_load_path,
 	  Fcons (Fsymbol_value (Qexec_directory, Qt),
-		 Fcons (rep_string_dup (REP_COMMON_EXEC_DIRECTORY),
+		 Fcons (rep_VAL (&common_exec),
 			Fnreverse(Fsymbol_value (Qdl_load_path, Qt)))));
 
     rep_INTERN_SPECIAL(after_load_alist);
@@ -2435,4 +2236,8 @@ rep_lispcmds_init(void)
     Fset (Qdl_load_reloc_now, Qnil);
 
     rep_INTERN_SPECIAL(load_filename);
+
+    rep_INTERN (_load_suffixes);
+    F_structure_set (rep_structure, Q_load_suffixes, Fcons (rep_VAL (&jl),
+							    rep_VAL (&jlc)));
 }
