@@ -1,6 +1,6 @@
 /* main.c -- Entry point for Jade
    Copyright (C) 1993, 1994 John Harper <john@dcs.warwick.ac.uk>
-   $Id: main.c,v 1.50 2000/04/03 17:28:50 john Exp $
+   $Id: main.c,v 1.57 2000/06/12 14:34:59 john Exp $
 
    This file is part of Jade.
 
@@ -195,6 +195,40 @@ get_main_options(char *prog_name, int *argc_p,
     return rep_TRUE;
 }
 
+static void
+check_configuration (int *stack_low)
+{
+    int stack_high;
+    int stack_dir = (&stack_high < stack_low) ? -1 : +1;
+
+    if (sizeof (rep_PTR_SIZED_INT) < sizeof(void *))
+    {
+	fprintf (stderr,
+	" ** error: --with-value-type is incorrect; it should be `%s'\n",
+		 (sizeof (int) >= sizeof (void *)) ? "int"
+		 : (sizeof (long) >= sizeof (void *)) ? "long"
+		 : (sizeof (rep_long_long) >= sizeof (void *)) ? "long long"
+		 : "<unknown>");
+	exit (10);
+    }
+
+    if (sizeof (rep_PTR_SIZED_INT) != rep_PTR_SIZED_INT_SIZEOF)
+    {
+	fprintf (stderr,
+	" ** error: --with-value-sizeof is incorrect; it should be %d\n",
+		 sizeof (rep_PTR_SIZED_INT));
+	exit (10);
+    }
+
+    if (stack_dir != STACK_DIRECTION)
+    {
+	fprintf (stderr,
+	" ** error: --with-stack-direction is incorrect; it should be %d\n",
+		 stack_dir);
+	exit (10);
+    }
+}
+
 /* Note that `argc' _must_ (I mean _must_!) be a pointer to the real
    argc on the stack frame of the outermost procedure */
 void
@@ -211,12 +245,8 @@ rep_init_from_dump(char *prog_name, int *argc, char ***argv,
 		   void (*sys_symbols)(void), void (*sys_usage)(void),
 		   char *dump_file)
 {
-    if(sizeof(rep_PTR_SIZED_INT) < sizeof(void *))
-    {
-	fputs("sizeof(rep_PTR_SIZED_INT) < sizeof(void *); aborting\n",
-	      stderr);
-	exit(10);
-    }
+    int dummy;
+    check_configuration (&dummy);
 
     if(!sys_memory_init())
 	exit(10);
@@ -232,10 +262,11 @@ rep_init_from_dump(char *prog_name, int *argc, char ***argv,
 	    rep_dumped_init (dump_file);
 
 	rep_symbols_init();
+	rep_structures_init ();
 	rep_numbers_init ();
 
-	rep_values_init();
 	rep_lisp_init();
+	rep_values_init();
 	rep_macros_init ();
 	rep_lispcmds_init();
 	rep_lispmach_init();
@@ -244,6 +275,8 @@ rep_init_from_dump(char *prog_name, int *argc, char ***argv,
 	rep_misc_init();
 	rep_streams_init();
 	rep_files_init();
+	rep_datums_init();
+	rep_fluids_init();
 	rep_sys_os_init();
 
 	/* XXX Assumes that argc is on the stack. I can't think of
@@ -269,18 +302,28 @@ rep_init_from_dump(char *prog_name, int *argc, char ***argv,
 repv
 rep_load_environment (repv file)
 {
+    repv old = rep_structure;
     repv res = Qnil;
-    rep_GC_root gc_file;
+    rep_GC_root gc_file, gc_old;
 
     rep_PUSHGC (gc_file, file);
+    rep_PUSHGC (gc_old, old);
+
+    /* 1. Do the rep bootstrap in the `rep' structure */
+    rep_structure = rep_default_structure;
     if (rep_dumped_non_constants != rep_NULL)
 	res = Feval (rep_dumped_non_constants);
     if (res != rep_NULL)
 	res = Fload (init_script, Qnil, Qnil, Qnil, Qnil);
+
+    rep_POPGC;
+    rep_structure = old;
+
+    /* 2. Do the caller-local bootstrap in the original structure */
     if (res != rep_NULL && rep_STRINGP(file))
 	res = Fload (file, Qnil, Qnil, Qnil, Qnil);
-    rep_POPGC;
 
+    rep_POPGC;
     return res;
 }
 
@@ -295,7 +338,7 @@ rep_kill(void)
 #endif
     rep_lispmach_kill();
     rep_db_kill();
-    rep_symbols_kill();
+    rep_tuples_kill();
     rep_values_kill();
     sys_memory_kill();
 }
@@ -327,7 +370,7 @@ rep_on_idle(long since_last_event)
 	res = rep_TRUE;
     else if(rep_data_after_gc > rep_idle_gc_threshold)
 	/* nothing was saved so try a GC */
-	Fgarbage_collect(Qt);
+	Fgarbage_collect (Qnil);
     else if(!called_hook && depth == 1)
     {
 	repv hook = Fsymbol_value(Qidle_hook, Qt);
@@ -394,8 +437,10 @@ rep_handle_input_exception(repv *result_p)
 	    (*rep_on_termination_fun)();
 	return rep_TRUE;
     }
+#if 0
     else if(rep_recurse_depth == 0)
 	rep_handle_error(Qno_catcher, rep_LIST_1(car));
+#endif
     else
     {
     unhandled:
