@@ -1,6 +1,6 @@
 /* misc.c -- Miscellaneous functions
    Copyright (C) 1993, 1994 John Harper <john@dcs.warwick.ac.uk>
-   $Id: misc.c,v 1.38 2000/03/29 23:00:37 john Exp $
+   $Id: misc.c,v 1.40 2000/05/25 13:49:53 john Exp $
 
    This file is part of Jade.
 
@@ -193,6 +193,17 @@ number of seconds since the start of the day (universal time).
     return rep_MAKE_TIME(time);
 }
 
+DEFUN("current-utime", Fcurrent_utime, Scurrent_utime, (void), rep_Subr0) /*
+::doc:current-utime::
+current-utime
+
+Return the current time in microseconds.
+::end:: */
+{
+    rep_long_long time = rep_utime ();
+    return rep_make_longlong_int (time);
+}
+
 DEFUN("fix-time", Ffix_time, Sfix_time, (repv time), rep_Subr1) /*
 ::doc:fix-time::
 fix-time TIMESTAMP
@@ -560,7 +571,7 @@ rep_misc_init(void)
     if (rep_beep_fun == 0)
 	rep_beep_fun = default_beep;
 
-    rep_INTERN_SPECIAL(operating_system);
+    rep_INTERN(operating_system);
 #ifdef rep_HAVE_UNIX
     rep_INTERN(unix);
     Fset (Qoperating_system, Qunix);
@@ -569,16 +580,17 @@ rep_misc_init(void)
     rep_INTERN_SPECIAL(process_environment);
     Fset (Qprocess_environment, Qnil);
 
-    rep_INTERN_SPECIAL(rep_version);
+    rep_INTERN(rep_version);
     Fset (Qrep_version, rep_VAL(&rep_version_string));
-    rep_INTERN_SPECIAL(rep_interface_id);
+    rep_INTERN(rep_interface_id);
     Fset (Qrep_interface_id, rep_VAL(rep_MAKE_INT(rep_INTERFACE)));
-    rep_INTERN_SPECIAL(rep_build_id);
+    rep_INTERN(rep_build_id);
     Fset (Qrep_build_id, rep_VAL(&build_id_string));
 
     rep_ADD_SUBR_INT(Sbeep);
     rep_ADD_SUBR(Scomplete_string);
     rep_ADD_SUBR(Scurrent_time);
+    rep_ADD_SUBR(Scurrent_utime);
     rep_ADD_SUBR(Sfix_time);
     rep_ADD_SUBR(Scurrent_time_string);
     rep_ADD_SUBR(Stime_later_p);
@@ -614,8 +626,8 @@ rep_misc_init(void)
 	rep_STR(up)[256] = 0;
 	rep_STR(down)[256] = 0;
 
-	rep_INTERN_SPECIAL(upcase_table);
-	rep_INTERN_SPECIAL(downcase_table);
+	rep_INTERN(upcase_table);
+	rep_INTERN(downcase_table);
 	Fset (Qupcase_table, up);
 	Fset (Qdowncase_table, down);
     }
@@ -628,7 +640,7 @@ rep_misc_init(void)
 	rep_STR(flatten)[10] = ' ';
 	rep_STR(flatten)[11] = 0;
 
-	rep_INTERN_SPECIAL(flatten_table);
+	rep_INTERN(flatten_table);
 	Fset (Qflatten_table, flatten);
     }
 }
