@@ -1,6 +1,6 @@
 /* readline.c -- wrap some readline functions when available
    Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
-   $Id: readline.c,v 1.7 2000/02/28 08:29:28 john Exp $
+   $Id: readline.c,v 1.10 2000/06/24 09:15:20 john Exp $
 
    This file is part of librep.
 
@@ -29,7 +29,6 @@
 # include <readline/history.h>
 #endif
 
-DEFSYM(readline, "readline");
 DEFSYM(rl_completion_generator, "rl-completion-generator");
 DEFSYM(boundp, "boundp");
 
@@ -44,7 +43,9 @@ completion_generator (char *word, int state)
 	repv fun = Fsymbol_value (Qrl_completion_generator, Qt);
 	if (Ffunctionp (fun) != Qnil)
 	{
-	    completions = rep_call_lisp1 (fun, rep_string_dup (word));
+	    completions = (rep_call_with_barrier
+			   (Ffuncall, rep_list_2 (fun, rep_string_dup (word)),
+			    rep_TRUE, 0, 0, 0));
 	}
 	else
 	{
@@ -194,9 +195,8 @@ DEFUN("readline", Freadline, Sreadline, (repv prompt_), rep_Subr1)
 repv
 rep_dl_init(void)
 {
-    rep_ADD_SUBR(Sreadline);
-    rep_INTERN(readline);
-    rep_INTERN_SPECIAL(rl_completion_generator);
+    repv tem;
+    rep_INTERN(rl_completion_generator);
     rep_INTERN(boundp);
     completions = Qnil;
     rep_mark_static (&completions);
@@ -205,5 +205,7 @@ rep_dl_init(void)
     rl_basic_quote_characters = "\"";
     init_bouncing_parens();
 #endif
-    return Qreadline;
+    tem = rep_push_structure ("readline");
+    rep_ADD_SUBR(Sreadline);
+    return rep_pop_structure (tem);
 }
