@@ -1,5 +1,5 @@
 /* rep.c -- read-eval-print front end
-   $Id: rep.c,v 1.12 2000/03/10 23:49:51 john Exp $ */
+   $Id: rep.c,v 1.15 2000/07/10 20:57:53 john Exp $ */
 
 #define _GNU_SOURCE
 
@@ -14,26 +14,30 @@
 static void
 usage (void)
 {
-    fputs ("\
+    fputs ("\n\
+    FILE		load the Lisp file FILE (from the cwd if possible,\n\
+			 implies --batch mode)\n\
+\n\
+    --call FUNCTION	call the Lisp function FUNCTION\n\
+    --f FUNCTION\n\
+\n\
+    --load FILE		load the file of Lisp forms called FILE\n\
+    -l FILE\n\
+\n\
+    --scheme FILE	load the file of Scheme forms called FILE\n\
+    -s FILE		 (implies --batch mode)\n\
+\n\
     --version		print version details\n\
     --no-rc		don't load rc or site-init files\n\
-    -f FUNCTION		call the Lisp function FUNCTION\n\
-    -l FILE		load the file of Lisp forms called FILE\n\
-    -q			quit\n\
-    FILE		load the Lisp file FILE (from the cwd if possible,\n\
-			  implies --batch mode)\n",
+    --quit, -q		terminate the interpreter process\n",
 	   stderr);
 }
 
-static repv
-inner_main (repv arg)
-{
-    return rep_load_environment (rep_string_dup ("rep"));
-}
-
 int
 main(int argc, char **argv)
 {
+    DEFSTRING (rep, "rep");
+
     char *prog_name = *argv++;
     argc--;
 
@@ -45,7 +49,8 @@ main(int argc, char **argv)
 	return 0;
     }
 
-    rep_call_with_barrier (inner_main, Qnil, rep_TRUE, 0, 0, 0);
+    rep_call_with_barrier (rep_load_environment, rep_VAL (&rep),
+			   rep_TRUE, 0, 0, 0);
 
     return rep_top_level_exit ();
 }
