@@ -1,6 +1,6 @@
 /* rep_lisp.h -- Data structures/objects for Lisp
    Copyright (C) 1993, 1994 John Harper <john@dcs.warwick.ac.uk>
-   $Id: rep_lisp.h,v 1.39 1999/11/25 23:23:13 john Exp $
+   $Id: rep_lisp.h,v 1.62 2000/07/15 11:41:05 john Exp $
 
    This file is part of Jade.
 
@@ -18,19 +18,13 @@
    along with Jade; see the file COPYING.  If not, write to
    the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
+/* library-private definitions are in repint.h */
+
 #ifndef REP_LISP_H
 #define REP_LISP_H
 
 #include <stdio.h>
 
-/* These numbers weren't just plucked from the air, they make the blocks
-   of objects fit as close as possible into powers of 2 sized blocks. */
-#define rep_CONSBLK_SIZE	510		/* ~4k */
-#define rep_SYMBOLBLK_SIZE	405		/* ~8k */
-
-/* The number of hash buckets in each rep_obarray, this is a prime number. */
-#define rep_OBSIZE		509
-
 /* Stringify X. Expands macros in X. */
 #define rep_QUOTE(x) rep_QUOTE__(x)
 #define rep_QUOTE__(x) #x
@@ -86,41 +80,7 @@ typedef unsigned rep_PTR_SIZED_INT repv;
 #define rep_VALUE_CONS_MARK_BIT	1
 #define rep_VALUE_IS_INT	2
 #define rep_VALUE_INT_SHIFT	2
-#define rep_CELL_ALIGNMENT	4
-
-/* Is repv V a cell type? */
-#define rep_CELLP(v)		(((v) & rep_VALUE_IS_INT) == 0)
-
-/* Is repv V an integer? */
-#define rep_INTP(v)		(!rep_CELLP(v))
-
-/* Convert a repv into a signed integer. */
-#define rep_INT(v)		(((rep_PTR_SIZED_INT)(v)) \
-				 >> rep_VALUE_INT_SHIFT)
-
-/* Convert a signed integer into a repv. */
-#define rep_MAKE_INT(x)		(((x) << rep_VALUE_INT_SHIFT) \
-				 | rep_VALUE_IS_INT)
-
-/* Bounds of the integer type */
-#define rep_LISP_INT_BITS	(rep_VALUE_BITS - rep_VALUE_INT_SHIFT)
-#define rep_LISP_MAX_INT	((rep_VALUE_CONST(1) \
-				  << (rep_LISP_INT_BITS - 1)) - 1)
-#define rep_LISP_MIN_INT	(-(rep_VALUE_CONST(1) \
-				   << (rep_LISP_INT_BITS - 1)))
-
-/* Store anything needing >24 bits (future expansion and all that),
-   in a cons cell, as one 24 bit, and one eight bit quantity. */
-#define rep_MAKE_LONG_INT(x) \
-    Fcons(rep_MAKE_INT((x) & 0x00ffffff), rep_MAKE_INT((x) >> 24))
-
-/* Convert a cons cell with two integers into a signed long int. */
-#define rep_LONG_INT(v) \
-    (rep_INT(rep_CAR(v)) | (rep_INT(rep_CDR(v)) << 24))
-
-/* True when V is a long integer. */
-#define rep_LONG_INTP(v) \
-    (rep_CONSP(v) && rep_INTP(rep_CAR(v)) && rep_INTP(rep_CDR(v)))
+#define rep_CELL_ALIGNMENT	rep_PTR_SIZED_INT_SIZEOF
 
 #if rep_CELL_ALIGNMENT <= rep_MALLOC_ALIGNMENT
   /* Allocate SIZE bytes of memory, aligned to NORMAL_ALIGNMENT */
@@ -141,17 +101,46 @@ typedef unsigned rep_PTR_SIZED_INT repv;
 #ifdef __GNUC__
 # define rep_ALIGN_CELL(d) d __attribute__ ((aligned (rep_CELL_ALIGNMENT)))
 #elif defined (__digital__) && defined (__unix__) && defined (__DECC)
-# if rep_CELL_ALIGNMENT >= 4
+# if rep_CELL_ALIGNMENT >= rep_PTR_SIZED_INT_SIZEOF
+   /* "the C compiler aligns an int (32 bits) on a 4-byte boundary and
+      a long (64 bits) on an 8-byte boundary" (Tru64 Programmer's Guide) */
 #  define rep_ALIGN_CELL(d) d
 # else
-#  error Cell alignment has changed (previously it was 4), and rep_lisp.h
-#  error must be updated for Tru64!
+#  error "You need to fix alignment for Tru64"
 # endif
 #else
 /* # warning Lets hope your compiler aligns to 4 byte boundaries.. */
 # define rep_ALIGN_CELL(d) d
 #endif
 
+/* Is repv V a cell type? */
+#define rep_CELLP(v)		(((v) & rep_VALUE_IS_INT) == 0)
+
+/* Is repv V a fixnum? */
+#define rep_INTP(v)		(!rep_CELLP(v))
+
+/* Convert a repv into a signed integer. */
+#define rep_INT(v)		(((rep_PTR_SIZED_INT)(v)) \
+				 >> rep_VALUE_INT_SHIFT)
+
+/* Convert a signed integer into a repv. */
+#define rep_MAKE_INT(x)		(((x) << rep_VALUE_INT_SHIFT) \
+				 | rep_VALUE_IS_INT)
+
+/* Bounds of the integer type */
+#define rep_LISP_INT_BITS	(rep_VALUE_BITS - rep_VALUE_INT_SHIFT)
+#define rep_LISP_MAX_INT	((rep_VALUE_CONST(1) \
+				  << (rep_LISP_INT_BITS - 1)) - 1)
+#define rep_LISP_MIN_INT	(-(rep_VALUE_CONST(1) \
+				   << (rep_LISP_INT_BITS - 1)))
+
+/* backwards compatibility */
+#define rep_MAKE_LONG_INT(x) rep_make_long_int(x)
+#define rep_LONG_INT(v) rep_get_long_int(v)
+#define rep_LONG_INTP(v) 						\
+    (rep_INTEGERP(v)							\
+     || (rep_CONSP(v) && rep_INTP(rep_CAR(v)) && rep_INTP(rep_CDR(v))))
+
 
 /* Structure of a cell */
 
@@ -187,8 +176,11 @@ typedef struct {
 /* Build a repv out of a pointer to a Lisp_Normal object */
 #define rep_VAL(x)		((repv)(x))
 
+/* Is V of cell8 type? */
+#define rep_CELL8P(v)		(rep_PTR(v)->car & rep_CELL_IS_8)
+
 /* Is V a cons? */
-#define rep_CELL_CONS_P(v)	(!(rep_PTR(v)->car & rep_CELL_IS_8))
+#define rep_CELL_CONS_P(v)	(!rep_CELL8P(v))
 
 /* Is V statically allocated? */
 #define rep_CELL_STATIC_P(v)	(rep_PTR(v)->car & rep_CELL_STATIC_BIT)
@@ -218,12 +210,6 @@ typedef struct {
     repv cdr;				/* low bit is GC mark */
 } rep_cons;
 
-/* Structure of cons allocation blocks */
-typedef struct rep_cons_block_struct {
-    struct rep_cons_block_struct *next;
-    rep_ALIGN_CELL(rep_cons cons[rep_CONSBLK_SIZE]);
-} rep_cons_block;
-
 #define rep_CONSP(v)	(rep_CELLP(v) && rep_CELL_CONS_P(v))
 
 /* Build a repv out of a pointer to a rep_cons object */
@@ -241,13 +227,9 @@ typedef struct rep_cons_block_struct {
 #define rep_GCDR(v)	(rep_CDR(v) & ~rep_VALUE_CONS_MARK_BIT)
 
 /* True if cons cell V is mutable (i.e. not read-only). */
-#ifdef rep_DUMPED
-# define rep_CONS_WRITABLE_P(v) \
-    (! (rep_CONS(v) >= &rep_dumped_cons_start \
-	&& rep_CONS(v) < &rep_dumped_cons_end))
-#else
-# define rep_CONS_WRITABLE_P(v) rep_TRUE
-#endif
+#define rep_CONS_WRITABLE_P(v) \
+    (! (rep_CONS(v) >= rep_dumped_cons_start \
+	&& rep_CONS(v) < rep_dumped_cons_end))
 
 
 /* Type data */
@@ -308,8 +290,8 @@ typedef struct rep_type_struct {
 #define rep_String	0x05
 #define rep_Compiled	0x07
 #define rep_Void	0x09
-#define rep_Process	0x0b
-#define rep_Var		0x0d
+#define rep_Reserved	0x0b
+#define rep_Number	0x0d
 #define rep_SF		0x0f
 #define rep_Subr0	0x11
 #define rep_Subr1	0x13
@@ -338,36 +320,62 @@ typedef struct rep_type_struct {
 /* true if V is of type T. */
 #define rep_TYPEP(v, t)	(rep_TYPE(v) == t)
 
+
+/* tuples, cells containing two values */
+
+typedef struct {
+    repv car;
+    repv a, b;
+} rep_tuple;
+
+#define rep_TUPLE(v)		((rep_tuple *) rep_PTR (v))
+
+
+/* Numbers (private defs in numbers.c) */
+
+/* Is V a non-fixnum number? */
+#define rep_NUMBERP(v)		rep_CELL8_TYPEP(v, rep_Number)
+
+/* Is V numeric? */
+#define rep_NUMERICP(v)		(rep_INTP(v) || rep_NUMBERP(v))
+
+/* bits 8-9 of car define number type (except when on freelist) */
+typedef rep_cell rep_number;
+
+/* these are in order of promotion */
+#define rep_NUMBER_INT		0	/* faked */
+#define rep_NUMBER_BIGNUM	0x100
+#define rep_NUMBER_RATIONAL	0x200
+#define rep_NUMBER_FLOAT	0x400
+
+#define rep_NUMBER_TYPE(v)	(((rep_number *)rep_PTR(v))->car & 0x700)
+#define rep_NUMBER_BIGNUM_P(v)	(rep_NUMBER_TYPE(v) & rep_NUMBER_BIGNUM)
+#define rep_NUMBER_RATIONAL_P(v) (rep_NUMBER_TYPE(v) & rep_NUMBER_RATIONAL)
+#define rep_NUMBER_FLOAT_P(v)	(rep_NUMBER_TYPE(v) & rep_NUMBER_FLOAT)
+
+#define rep_NUMERIC_TYPE(v) \
+    (rep_INTP(v) ? rep_NUMBER_INT : rep_NUMBER_TYPE(v))
+
+#define rep_INTEGERP(v) \
+    (rep_INTP(v) || (rep_NUMBERP(v) && rep_NUMBER_BIGNUM_P(v)))
+
 
 /* Strings */
 
 typedef struct rep_string_struct {
     /* Bits 0->7 are standard cell8 defines. Bits 8->31 store the length
-       of the string.
-
-       This means that strings can't contain more than 2^24-1 bytes
-       (thats about 16.7MB) */
+       of the string. This means that strings can't contain more than
+       2^24-1 bytes (thats about 16.7MB) */
     repv car;
-    struct rep_string_struct *next;
-
-    /* This is inefficient since the data for dynamically allocated
-       strings always follows immediately. But there's no way to do the
-       same for statically allocated strings. [I used to use some
-       assembler magic for this purpose, but it's less than portable] */
 
+    /* Pointer to the (zero-terminated) characters */
     u_char *data;
-
-    /* in a dynamically allocated string, data follows */
 } rep_string;
 
 #define rep_STRING_LEN_SHIFT	8
 #define rep_MAX_STRING \
     ((rep_VALUE_CONST(1) << (rep_VALUE_BITS - rep_STRING_LEN_SHIFT)) - 1)
 
-/* The number of bytes that need to be allocated for a string cell
-   containg X string bytes (including terminating zero). */
-#define rep_DSTRING_SIZE(x) 	(sizeof(rep_string) + (x))
-
 #define rep_STRINGP(v)		rep_CELL8_TYPEP(v, rep_String)
 #define rep_STRING(v)		((rep_string *) rep_PTR(v))
 
@@ -381,11 +389,12 @@ typedef struct rep_string_struct {
 
 /* Define a variable V, containing a static string S. This must be cast
    to a repv via the rep_VAL() macro when using. */
-#define DEFSTRING(v, s) 				\
-    rep_ALIGN_CELL(static rep_string v)			\
-	= { ((sizeof(s) - 1) << rep_STRING_LEN_SHIFT)	\
-	    | rep_CELL_STATIC_BIT | rep_String, 0,	\
-	    (u_char *)s }
+#define DEFSTRING(v, s)					\
+    rep_ALIGN_CELL(const static rep_string v) = {	\
+	((sizeof(s) - 1) << rep_STRING_LEN_SHIFT)	\
+	| rep_CELL_STATIC_BIT | rep_String,		\
+	(u_char *)s					\
+    }
 
 #define rep_STR(v)	(rep_STRING(v)->data)
 
@@ -395,22 +404,13 @@ typedef struct rep_string_struct {
 
 /* Symbols */
 
-/* Symbol object, each symbol has 4 basic attributes, a name, its value
-   as a variable, its value as a function and a property-list.
-   Symbols are generally stored in hash tables (rep_obarray) with collisions
-   chained from the `QNext' field.  */
+/* symbol object, actual allocated as a tuple */
 typedef struct {
     repv car;				/* bits 8->11 are flags */
     repv next;				/* next symbol in rep_obarray bucket */
     repv name;
-    repv value;
-    repv prop_list;
 } rep_symbol;
 
-/* This bit set in car means that the value is a constant, and therefore
-   can't be modified. */
-#define rep_SF_CONSTANT	(1 << (rep_CELL8_TYPE_BITS + 0))
-
 /* Means that the symbol's value may be in some form of local storage,
    if so then that occurrence takes precedence. */
 #define rep_SF_LOCAL 	(1 << (rep_CELL8_TYPE_BITS + 1))
@@ -438,12 +438,6 @@ typedef struct {
 /* Set when the variable has been defvar'd */
 #define rep_SF_DEFVAR	(1 << (rep_CELL8_TYPE_BITS + 7))
 
-/* Symbol allocation blocks */
-typedef struct rep_symbol_block_struct {
-    struct rep_symbol_block_struct *next;
-    rep_ALIGN_CELL(rep_symbol symbols[rep_SYMBOLBLK_SIZE]);
-} rep_symbol_block;
-
 #define rep_SYM(v)		((rep_symbol *)rep_PTR(v))
 #define rep_SYMBOLP(v)		rep_CELL8_TYPEP(v, rep_Symbol)
 
@@ -470,13 +464,7 @@ typedef struct rep_vector_struct {
 
 #define rep_VECTORP(v)		rep_CELL8_TYPEP(v, rep_Vector)
 
-#ifdef rep_DUMPED
-# define rep_VECTOR_WRITABLE_P(v)				\
-    (!(rep_VECT(v) >= (rep_vector *)&rep_dumped_text_start	\
-       && rep_VECT(v) < (rep_vector *)&rep_dumped_text_end))
-#else
-# define rep_VECTOR_WRITABLE_P(v) rep_TRUE
-#endif
+#define rep_VECTOR_WRITABLE_P(v) (!rep_CELL_STATIC_P(v))
 
 
 /* Compiled Lisp functions; this is a vector. Some of these definitions
@@ -495,7 +483,7 @@ typedef struct rep_vector_struct {
 #define rep_COMPILED_CONSTANTS(v) rep_VECTI(v, 2)
 
 /* Fourth is an integer: stack usage */
-#define rep_COMPILED_STACK(v)	rep_INT(rep_VECTI(v, 3))
+#define rep_COMPILED_STACK(v)	rep_VECTI(v, 3)
 
 #define rep_COMPILED_MIN_SLOTS	4
 
@@ -583,38 +571,29 @@ typedef struct {
 #define rep_SUBR4FUN(v)	(rep_SUBR(v)->fun.fun4)
 #define rep_SUBR5FUN(v)	(rep_SUBR(v)->fun.fun5)
 #define rep_SUBRNFUN(v)	(rep_SUBR(v)->fun.fun1)
-#define rep_SFFUN(v)	(rep_SUBR(v)->fun.fun1)
-#define rep_VARFUN(v)	(rep_SUBR(v)->fun.fun1)
+#define rep_SFFUN(v)	(rep_SUBR(v)->fun.fun2)
 
 
 /* Closures */
 
 typedef struct rep_funarg_struct {
     repv car;
-    struct rep_funarg_struct *next;
     repv fun;
     repv name;
     repv env;
-    repv special_env;
-    repv fh_env;			/* file handlers */
+    repv structure;
 } rep_funarg;
 
 #define rep_FUNARG(v) ((rep_funarg *)rep_PTR(v))
 #define rep_FUNARGP(v) (rep_CELL8_TYPEP(v, rep_Funarg))
 
-#define rep_USE_FUNARG(f)				\
-    do {						\
-	rep_env = rep_FUNARG(f)->env;			\
-	rep_special_env = rep_FUNARG(f)->special_env;	\
-	rep_fh_env = rep_FUNARG(f)->fh_env;		\
-    } while (0)
+#define rep_FUNARG_WRITABLE_P(v) (!rep_CELL_STATIC_P(v))
 
-#define rep_USE_DEFAULT_ENV			\
-    do {					\
-	rep_env = Qt;				\
-	rep_special_env = Fcons (Qnil, Qt);	\
-	rep_fh_env = Qt;			\
-    } while (0)
+
+/* Guardians */
+
+#define rep_GUARDIAN(v)		((rep_guardian *) rep_PTR(v))
+#define rep_GUARDIANP(v)	rep_CELL16_TYPEP(v, rep_guardian_type)
 
 
 /* Other definitions */
@@ -767,38 +746,6 @@ typedef struct rep_gc_n_roots {
 
 #endif
 
-
-/* More other stuff */
-
-/* Keeps a backtrace of all lisp functions called. */
-struct rep_Call {
-    struct rep_Call *next;
-    repv fun;
-    repv args;
-    /* t if `args' is list of *evalled* arguments.  */
-    repv args_evalled_p;
-    repv saved_env;
-    repv saved_special_env;
-    repv saved_fh_env;
-};
-
-#define rep_PUSH_CALL(lc)		\
-    do {				\
-	(lc).saved_env = rep_env;	\
-	(lc).saved_special_env = rep_special_env; \
-	(lc).saved_fh_env = rep_fh_env;	\
-	(lc).next = rep_call_stack;	\
-	rep_call_stack = &(lc);		\
-    } while (0)
-
-#define rep_POP_CALL(lc)		\
-    do {				\
-	rep_env = (lc).saved_env;	\
-	rep_special_env = (lc).saved_special_env; \
-	rep_fh_env = (lc).saved_fh_env;	\
-	rep_call_stack = (lc).next;	\
-    } while (0)
-
 
 /* Macros for declaring functions */
 
@@ -824,10 +771,13 @@ struct rep_Call {
     repv fsym args
 
 /* Add a subroutine */    
-#define rep_ADD_SUBR(subr) rep_add_subr(&subr)
+#define rep_ADD_SUBR(subr) rep_add_subr(&subr, rep_TRUE)
+
+/* Add a non-exported subroutine */
+#define rep_ADD_INTERNAL_SUBR(subr) rep_add_subr(&subr, rep_FALSE)
 
 /* Add an interactive subroutine */    
-#define rep_ADD_SUBR_INT(subr) rep_ADD_SUBR(subr)
+#define rep_ADD_SUBR_INT(subr) rep_add_subr(&subr, rep_TRUE)
 
 /* Declare a symbol stored in variable QX. */
 #define DEFSYM(x, name) \
@@ -841,7 +791,8 @@ struct rep_Call {
 #define rep_INTERN_SPECIAL(x) 					\
     do {							\
 	rep_intern_static (& Q ## x, rep_VAL(& str_ ## x));	\
-	rep_SYM(Q ## x)->car |= rep_SF_SPECIAL | rep_SF_DEFVAR;	\
+	Fmake_variable_special (Q ## x);			\
+	rep_SYM(Q ## x)->car |= rep_SF_DEFVAR;			\
     } while (0)
 
 /* Add an error string called err_X for symbol stored in QX */
@@ -870,6 +821,16 @@ struct rep_Call {
 
 /* Macros for interrupt handling */
 
+#define rep_MAY_YIELD						\
+    do {							\
+	if (rep_pending_thread_yield && rep_thread_lock == 0)	\
+	    Fthread_yield ();					\
+    } while (0)
+
+#define rep_FORBID rep_thread_lock++
+#define rep_PERMIT rep_thread_lock--
+#define rep_PREEMPTABLE_P (rep_thread_lock <= 0)
+
 /* rep_TEST_INT is called before testing rep_INTERRUPTP, if necessary the
    target operating system will define it to be something useful.
    There's also a variant rep_TEST_INT_SLOW that should be used by code that
@@ -881,6 +842,7 @@ struct rep_Call {
 	if(++rep_test_int_counter > rep_test_int_period) { 	\
 	    (*rep_test_int_fun)();				\
 	    rep_test_int_counter = 0;				\
+	    rep_pending_thread_yield = rep_TRUE;		\
 	}							\
     } while(0)
 
@@ -888,6 +850,8 @@ struct rep_Call {
     do {				\
 	(*rep_test_int_fun)();		\
 	rep_test_int_counter = 0;	\
+	if (!rep_INTERRUPTP)		\
+	    Fthread_yield ();		\
     } while(0)
 
 #else /* !rep_TEST_INT */
@@ -913,22 +877,4 @@ struct rep_Call {
 
 #define rep_TIMEP(v) rep_CONSP(v)
 
-
-/* Dumped Lisp code */
-
-#ifdef rep_DUMPED
-
-extern rep_PTR_SIZED_INT rep_dumped_text_start, rep_dumped_data_start;
-extern rep_PTR_SIZED_INT rep_dumped_text_end, rep_dumped_data_end;
-
-extern rep_string rep_dumped_strings_start, rep_dumped_strings_end;
-extern rep_cons rep_dumped_cons_start, rep_dumped_cons_end;
-extern rep_symbol rep_dumped_symbols_start, rep_dumped_symbols_end;
-extern rep_vector rep_dumped_vectors_start, rep_dumped_vectors_end;
-extern rep_vector rep_dumped_bytecode_start, rep_dumped_bytecode_end;
-
-#define rep_DUMPED_SYM_NIL ((&rep_dumped_symbols_end)-1)	/* trust me */
-
-#endif /* rep_DUMPED */
-
 #endif /* REP_LISP_H */
