@@ -1,6 +1,6 @@
 /* repint_subrs.h -- library-local prototypes
    Copyright (C) 1993, 1994 John Harper <john@dcs.warwick.ac.uk>
-   $Id: repint_subrs.h,v 1.13 2000/03/19 21:20:46 john Exp $
+   $Id: repint_subrs.h,v 1.30 2000/07/14 11:08:54 john Exp $
 
    This file is part of Jade.
 
@@ -24,6 +24,9 @@
 /* from continuations.c */
 extern void rep_continuations_init (void);
 
+/* from datums.c */
+extern void rep_datums_init (void);
+
 /* from files.c */
 extern void rep_files_init(void);
 extern void rep_files_kill(void);
@@ -34,11 +37,17 @@ extern void rep_mark_regexp_data(void);
 extern void rep_find_init(void);
 extern void rep_find_kill(void);
 
+/* from fluids.c */
+extern void rep_fluids_init (void);
+
 /* from lisp.c */
+extern repv rep_scm_t, rep_scm_f;
 extern repv rep_readl(repv, int *);
+extern repv rep_bind_lambda_list_1 (repv lambdaList, repv *args, int nargs,
+				    repv (*binder)(repv, repv, repv));
 extern repv rep_bind_lambda_list(repv lambdaList, repv argList,
 				 rep_bool eval_args, rep_bool eval_in_env);
-extern repv rep_eval_lambda(repv, repv, rep_bool, rep_bool);
+extern repv rep_eval (repv form, repv tail_posn);
 extern void rep_lisp_prin(repv, repv);
 extern void rep_string_princ(repv, repv);
 extern void rep_string_print(repv, repv);
@@ -47,11 +56,17 @@ extern repv rep_copy_list(repv);
 extern rep_bool rep_compare_error(repv error, repv handler);
 extern void rep_lisp_init(void);
 extern rep_bool rep_single_step_flag;
+extern repv rep_call_lispn (repv fun, int argc, repv *argv);
 
 /* from lispcmds.c */
+extern rep_xsubr Slambda;
+extern repv Fcall_with_exception_handler (repv, repv);
 extern void rep_lispcmds_init(void);
 
 /* from lispmach.c */
+extern repv Qrun_byte_code;
+extern repv Frun_byte_code(repv code, repv consts, repv stkreq);
+extern repv rep_apply_bytecode (repv subr, int nargs, repv *args);
 extern void rep_lispmach_init(void);
 extern void rep_lispmach_kill(void);
 
@@ -72,6 +87,7 @@ extern void rep_misc_init(void);
 /* from numbers.c */
 extern repv rep_parse_number (char *buf, u_int len, u_int radix,
 			      int sign, u_int type);
+extern repv rep_foldl (repv args, repv (*op)(repv, repv));
 extern void rep_numbers_init (void);
 
 /* from regsub.c */
@@ -81,15 +97,36 @@ extern int rep_default_regsublen(int, rep_regsubs *, char *, void *);
 /* from streams.c */
 extern void rep_streams_init(void);
 
+/* from structures.c */
+extern repv rep_default_structure, rep_specials_structure;
+extern rep_struct_node *rep_search_imports (rep_struct *s, repv var);
+extern repv F_make_structure (repv, repv, repv, repv);
+extern repv F_structure_ref (repv, repv);
+extern repv F_structure_set (repv, repv, repv);
+extern repv F_structure_bound_p (repv, repv);
+extern repv F_external_structure_ref (repv, repv);
+extern repv F_intern_structure (repv);
+extern repv rep_get_initial_special_value (repv sym);
+extern void rep_pre_structures_init (void);
+extern void rep_structures_init (void);
+
 /* from symbols.c */
 extern int rep_pre_symbols_init(void);
 extern void rep_symbols_init(void);
-extern void rep_symbols_kill(void);
-extern int rep_allocated_symbols, rep_used_symbols;
+extern int rep_allocated_funargs, rep_used_funargs;
+
+/* from tuples.c */
+extern int rep_allocated_tuples, rep_used_tuples;
+extern void rep_sweep_tuples (void);
+extern void rep_tuples_kill(void);
 
 /* from values.c */
 extern int rep_type_cmp(repv, repv);
 extern int rep_ptr_cmp(repv, repv);
+extern rep_cons_block *rep_cons_block_chain;
+extern rep_cons *rep_cons_freelist;
+extern int rep_allocated_cons, rep_used_cons;
+extern rep_cons *rep_allocate_cons (void);
 extern void rep_cons_free(repv);
 extern void rep_pre_values_init (void);
 extern void rep_values_init(void);
@@ -98,6 +135,11 @@ extern void rep_dumped_init(char *file);
 
 #ifdef rep_HAVE_UNIX
 
+/* from unix_dl.c */
+extern repv rep_open_dl_library(repv file_name);
+extern void rep_mark_dl_data(void);
+extern void rep_kill_dl_libraries(void);
+
 /* from unix_main.c */
 extern repv rep_user_login_name(void);
 extern repv rep_user_full_name(void);
