@@ -1,5 +1,5 @@
 /* repsdbm.c -- rep wrapper to libsdbm
-   $Id: repsdbm.c,v 1.10 2000/02/28 08:29:28 john Exp $ */
+   $Id: repsdbm.c,v 1.11 2000/04/07 16:31:35 john Exp $ */
 
 #define _GNU_SOURCE
 
@@ -27,7 +27,6 @@ typedef struct rep_dbm_struct {
 
 static rep_dbm *dbm_chain;
 
-DEFSYM(sdbm, "sdbm");
 DEFSYM(insert, "insert");
 DEFSYM(replace, "replace");
 
@@ -249,10 +248,15 @@ dbm_compare (repv v1, repv v2)
 repv
 rep_dl_init (void)
 {
+    repv tem;
     dbm_type = rep_register_new_type ("sdbm", dbm_compare,
 				      dbm_print, dbm_print,
 				      dbm_sweep, dbm_mark,
 				      0, 0, 0, 0, 0, 0, 0);
+    rep_INTERN (insert);
+    rep_INTERN (replace);
+
+    tem = rep_push_structure ("sdbm");
     rep_ADD_SUBR(Ssdbm_open);
     rep_ADD_SUBR(Ssdbm_close);
     rep_ADD_SUBR(Ssdbm_fetch);
@@ -263,9 +267,5 @@ rep_dl_init (void)
     rep_ADD_SUBR(Ssdbm_rdonly);
     rep_ADD_SUBR(Ssdbm_error);
     rep_ADD_SUBR(Ssdbmp);
-
-    rep_INTERN (sdbm);
-    rep_INTERN (insert);
-    rep_INTERN (replace);
-    return Qsdbm;
+    return rep_pop_structure (tem);
 }
