@@ -1,6 +1,6 @@
 /* symbols.c -- Lisp symbol handling
    Copyright (C) 1993, 1994 John Harper <john@dcs.warwick.ac.uk>
-   $Id: symbols.c,v 1.42 2000/04/03 17:28:41 john Exp $
+   $Id: symbols.c,v 1.62 2000/07/15 11:41:19 john Exp $
 
    This file is part of Jade.
 
@@ -50,18 +50,19 @@ char *alloca ();
 /* The number of hash buckets in each rep_obarray, this is a prime number. */
 #define rep_OBSIZE		509
 
-#define rep_SYMBOLBLK_SIZE	405		/* ~8k */
+#define rep_FUNARGBLK_SIZE	204		/* ~4k */
 
-/* Symbol allocation blocks */
-typedef struct rep_symbol_block_struct {
-    struct rep_symbol_block_struct *next;
-    rep_ALIGN_CELL(rep_symbol symbols[rep_SYMBOLBLK_SIZE]);
-} rep_symbol_block;
+/* Closure allocation blocks */
+typedef struct rep_funarg_block_struct {
+    struct rep_funarg_block_struct *next;
+    rep_ALIGN_CELL(rep_funarg data[rep_FUNARGBLK_SIZE]);
+} rep_funarg_block;
 
 /* Main storage of symbols.  */
 repv rep_obarray;
 
-static rep_funarg *funargs;
+/* Plist storage */
+static repv plist_structure;
 
 DEFSYM(nil, "nil");
 DEFSYM(t, "t");
@@ -81,9 +82,13 @@ repv rep_void_value = rep_VAL(&void_object);
    It can be any Lisp object which isn't a symbol.  */
 #define OB_NIL rep_VAL(&void_object)
 
-static rep_symbol_block *symbol_block_chain;
-static rep_symbol *symbol_freelist;
-int rep_allocated_symbols, rep_used_symbols;
+/* Used to mark lexical bindings */
+rep_ALIGN_CELL(static rep_cell lextag) = { rep_Void };
+#define LEXTAG rep_VAL(&lextag)
+
+static rep_funarg_block *funarg_block_chain;
+static rep_funarg *funarg_freelist;
+int rep_allocated_funargs, rep_used_funargs;
 
 
 /* Symbol management */
@@ -96,62 +101,13 @@ Returns a new, uninterned, symbol with print-name NAME. It's value and
 function definition are both void and it has a nil property-list.
 ::end:: */
 {
-    repv sym;
     rep_DECLARE1(name, rep_STRINGP);
-    if(!symbol_freelist)
-    {
-	rep_symbol_block *sb = rep_ALLOC_CELL(sizeof(rep_symbol_block));
-	if(sb)
-	{
-	    int i;
-	    rep_allocated_symbols += rep_SYMBOLBLK_SIZE;
-	    sb->next = symbol_block_chain;
-	    symbol_block_chain = sb;
-	    for(i = 0; i < (rep_SYMBOLBLK_SIZE - 1); i++)
-		sb->symbols[i].next = rep_VAL(&sb->symbols[i + 1]);
-	    sb->symbols[i].next = rep_VAL(symbol_freelist);
-	    symbol_freelist = sb->symbols;
-	}
-    }
-    if((sym = rep_VAL(symbol_freelist)))
-    {
-	symbol_freelist = rep_SYM(rep_SYM(sym)->next);
-	rep_SYM(sym)->next = rep_NULL;
-	rep_SYM(sym)->car = rep_Symbol;
-	rep_SYM(sym)->name = name;
-	rep_SYM(sym)->value = rep_void_value;
-	rep_SYM(sym)->prop_list = Qnil;
-	rep_used_symbols++;
-	rep_data_after_gc += sizeof(rep_symbol);
-    }
-    return(sym);
+    return rep_make_tuple (rep_Symbol, rep_NULL, name);
 }
 
 static void
 symbol_sweep(void)
 {
-    rep_symbol_block *sb = symbol_block_chain;
-    symbol_freelist = NULL;
-    rep_used_symbols = 0;
-    while(sb)
-    {
-	int i;
-	rep_symbol_block *nxt = sb->next;
-	for(i = 0; i < rep_SYMBOLBLK_SIZE; i++)
-	{
-	    if(!rep_GC_CELL_MARKEDP(rep_VAL(&sb->symbols[i])))
-	    {
-		sb->symbols[i].next = rep_VAL(symbol_freelist);
-		symbol_freelist = &sb->symbols[i];
-	    }
-	    else
-	    {
-		rep_GC_CLR_CELL(rep_VAL(&sb->symbols[i]));
-		rep_used_symbols++;
-	    }
-	}
-	sb = nxt;
-    }
     /* Need to clear mark bits of dumped symbols, since they're mutable */
     if (rep_dumped_symbols_start != rep_dumped_symbols_end)
     {
@@ -159,10 +115,7 @@ symbol_sweep(void)
 	for(s = rep_dumped_symbols_start; s < rep_dumped_symbols_end; s++)
 	{
 	    if(rep_GC_CELL_MARKEDP(rep_VAL(s)))
-	    {
 		rep_GC_CLR_CELL(rep_VAL(s));
-		rep_used_symbols++;
-	    }
 	}
     }
 }
@@ -190,64 +143,62 @@ symbol_princ(repv strm, repv obj)
 static void
 symbol_print(repv strm, repv obj)
 {
-    u_char *s = rep_STR(rep_SYM(obj)->name);
-    u_char c;
-    rep_bool all_digits = isdigit (*s);
+    /* output a maximum of 2n chars for a symbol name of length n */
+    u_char *buf = alloca (rep_STRING_LEN (rep_SYM (obj)->name) * 2);
+    register u_char *out = buf;
+    register u_char *s;
+    rep_bool seen_digit = rep_FALSE;
 
-    while((c = *s++))
+    s = rep_STR (rep_SYM (obj)->name);
+    switch (*s++)
     {
-	switch(c)
+    case '0': case '1': case '2': case '3': case '4':
+    case '5': case '6': case '7': case '8': case '9':
+	seen_digit = rep_TRUE;
+
+    case '-': case '+': case '.':
+
+    pass1:
+	switch (*s++)
 	{
+	case 0:
+	    if (seen_digit)
+		*out++ = '\\';
+	    break;
+
+	case '0': case '1': case '2': case '3': case '4':
+	case '5': case '6': case '7': case '8': case '9':
+	    seen_digit = rep_TRUE;
+	case '/': case '.':
+	    goto pass1;
+	}
+    }
+
+    s = rep_STR (rep_SYM (obj)->name);
+    while (1)
+    {
+	u_char c = *s++;
+	switch (c)
+	{
+	case 0:
+	    goto out;
+
 	case ' ': case '\t': case '\n': case '\f':
 	case '(': case ')': case '[': case ']':
 	case '\'': case '"': case ';': case '\\':
-	case '|':
-	    rep_stream_putc(strm, (int)'\\');
-	    all_digits = rep_FALSE;
-	    break;
-
-	case '.': case '/': case 'e': case 'E':
-	    if (all_digits)
-	    {
-		rep_stream_putc (strm, (int) '\\');
-		all_digits = rep_FALSE;
-	    }
+	case '|': case '#': case ',': case '`':
+	    *out++ = '\\';
 	    break;
 
 	default:
-	    if (!isdigit (c))
-		all_digits = rep_FALSE;
-	    if(iscntrl(c))
-		rep_stream_putc(strm, (int)'\\');
+	    if (iscntrl (c))
+		*out++ = '\\';
 	    break;
 	}
-	rep_stream_putc(strm, (int)c);
+	*out++ = c;
     }
-}
-
-repv
-rep_add_subr(rep_xsubr *subr)
-{
-    repv sym = Fintern(subr->name, rep_obarray);
-    if(sym)
-    {
-	rep_SYM(sym)->value = rep_VAL(subr);
-	rep_SYM(sym)->car |= rep_SF_DEFVAR;
-    }
-    return(sym);
-}
-
-repv
-rep_add_const_num(repv name, long num)
-{
-    repv sym = Fintern(name, rep_obarray);
-    if(sym)
-    {
-	rep_SYM(sym)->car |= rep_SF_SPECIAL;
-	rep_SYM(sym)->value = rep_MAKE_INT(num);
-	rep_SYM(sym)->car |= rep_SF_CONSTANT;
-    }
-    return(sym);
+out:
+    rep_stream_puts (strm, buf, out - buf, rep_FALSE);
 }
 
 void
@@ -395,18 +346,31 @@ Return a functional object which makes the closure of FUNCTION and the
 current environment.
 ::end:: */
 {
-    rep_funarg *f = rep_ALLOC_CELL (sizeof (rep_funarg));
+    rep_funarg *f;
+    if(!funarg_freelist)
+    {
+	rep_funarg_block *sb = rep_ALLOC_CELL(sizeof(rep_funarg_block));
+	if(sb)
+	{
+	    int i;
+	    rep_allocated_funargs += rep_FUNARGBLK_SIZE;
+	    sb->next = funarg_block_chain;
+	    funarg_block_chain = sb;
+	    for(i = 0; i < (rep_FUNARGBLK_SIZE - 1); i++)
+		sb->data[i].car = rep_VAL(&sb->data[i + 1]);
+	    sb->data[i].car = rep_VAL(funarg_freelist);
+	    funarg_freelist = sb->data;
+	}
+    }
+
+    f = funarg_freelist;
+    funarg_freelist = rep_FUNARG (f->car);
     rep_data_after_gc += sizeof (rep_funarg);
     f->car = rep_Funarg;
-    if (rep_bytecode_interpreter != Fjade_byte_code)
-	f->car |= rep_FF_NO_BYTE_CODE;
     f->fun = fun;
     f->name = name;
     f->env = rep_env;
-    f->special_env = rep_special_env;
-    f->fh_env = rep_fh_env;
-    f->next = funargs;
-    funargs = f;
+    f->structure = rep_structure;
     return rep_VAL (f);
 }
 
@@ -435,6 +399,30 @@ Set the function value in the closure FUNARG to FUNCTION.
     return fun;
 }
 
+DEFUN("closure-structure", Fclosure_structure,
+      Sclosure_structure, (repv funarg), rep_Subr1) /*
+::doc:closure-function::
+closure-function FUNARG
+
+Return the structure associated with the closure FUNARG.
+::end:: */
+{
+    rep_DECLARE1(funarg, rep_FUNARGP);
+    return rep_FUNARG(funarg)->structure;
+}
+
+DEFUN("closure-name", Fclosure_name,
+      Sclosure_name, (repv funarg), rep_Subr1) /*
+::doc:closure-name::
+closure-name FUNARG
+
+Return the name associated with the closure FUNARG.
+::end:: */
+{
+    rep_DECLARE1(funarg, rep_FUNARGP);
+    return rep_FUNARG(funarg)->name;
+}
+
 DEFUN("closurep", Fclosurep, Sclosurep, (repv arg), rep_Subr1) /*
 ::doc:closurep::
 funargp ARG
@@ -445,102 +433,75 @@ Returns t if ARG is a closure
     return rep_FUNARGP(arg) ? Qt : Qnil;
 }
 
-DEFUN("save-environment", Fsave_environment,
-      Ssave_environment, (repv args), rep_SF) /*
-::doc:save-environment::
-save-environment FORMS...
-::end:: */
-{
-    repv ret;
-    struct rep_Call lc;
-    lc.fun = Qnil;
-    lc.args = Qnil;
-    lc.args_evalled_p = Qnil;
-    rep_PUSH_CALL(lc);
-    ret = Fprogn (args);
-    rep_POP_CALL(lc);
-    return ret;
-}
-
-DEFUN("set-environment", Fset_environment,
-      Sset_environment, (repv env), rep_Subr1) /*
-::doc:set-environment::
-set-variable-environment ENV
-::end:: */
-{
-    if (rep_call_stack != 0)
-    {
-	if (rep_LISTP (env))
-	{
-	    repv tem;
-	    rep_call_stack->saved_env = env;
-	    tem = Fassq (Qjade_byte_code, env);
-	    if (tem && tem != Qnil)
-	    {
-		tem = rep_CDR (tem);
-		if (rep_CELL8_TYPEP (tem, rep_Subr3))
-		    rep_bytecode_interpreter = rep_SUBR4FUN (tem);
-		else
-		    rep_bytecode_interpreter = 0;
-	    }
-	}
-    }
-    return Qt;
-}
-
 DEFUN("set-special-environment", Fset_special_environment,
-      Sset_special_environment, (repv env), rep_Subr1) /*
+      Sset_special_environment, (repv env, repv structure), rep_Subr2) /*
 ::doc:set-special-environment::
-set-special-environment ENV
+set-special-environment ENV STRUCTURE
 ::end:: */
 {
-    if (rep_call_stack != 0)
-    {
-	if (rep_LISTP (env))
-	    rep_call_stack->saved_special_env = Fcons (Qnil, env);
-    }
+    rep_DECLARE2 (structure, rep_STRUCTUREP);
+    rep_STRUCTURE (structure)->special_env = env;
     return Qt;
 }
 
 static void
 funarg_sweep (void)
 {
-    rep_funarg *f = funargs;
-    funargs = 0;
-    while (f != 0)
+    rep_funarg_block *sb = funarg_block_chain;
+    funarg_freelist = NULL;
+    rep_used_funargs = 0;
+    while(sb)
     {
-	rep_funarg *next = f->next;
-	if (!rep_GC_CELL_MARKEDP(rep_VAL(f)))
-	    rep_FREE_CELL(f);
-	else
+	int i;
+	rep_funarg_block *nxt = sb->next;
+	for(i = 0; i < rep_FUNARGBLK_SIZE; i++)
 	{
-	    rep_GC_CLR_CELL(rep_VAL(f));
-	    f->next = funargs;
-	    funargs = f;
+	    /* if on the freelist then the CELL_IS_8 bit
+	       will be unset (since the pointer is long aligned) */
+	    if (rep_CELL_CONS_P(rep_VAL(&sb->data[i]))
+		|| !rep_GC_CELL_MARKEDP(rep_VAL(&sb->data[i])))
+	    {
+		sb->data[i].car = rep_VAL(funarg_freelist);
+		funarg_freelist = &sb->data[i];
+	    }
+	    else
+	    {
+		rep_GC_CLR_CELL(rep_VAL(&sb->data[i]));
+		rep_used_funargs++;
+	    }
 	}
-	f = next;
+	sb = nxt;
     }
 }
 
-/* Returns (SYM . VALUE) if a lexical binding. Returns t if the actual
-   value is in the symbol's function slot */
+/* Returns (SYM . VALUE) if a lexical binding, or nil */
 static repv
 search_environment (repv sym)
 {
-    register repv env = rep_env;
-    while (rep_CONSP(env) && rep_CAR(rep_CAR(env)) != sym)
-	env = rep_CDR(env);
-    return rep_CONSP(env) ? rep_CAR(env) : env;
+    register repv env;
+    for (env = rep_env; env != Qnil; env = rep_CDR (env))
+    {
+	if (rep_CONSP (rep_CAR (env))
+	    && rep_CAAR(env) == LEXTAG
+	    && rep_CADAR(env) == sym)
+	{
+	    return rep_CDAR (env);
+	}
+    }
+    return Qnil;
 }
 
-/* this is also in lispmach.c */
+/* this is also in lispmach.c and fluids.c */
 static inline repv
 inlined_search_special_bindings (repv sym)
 {
-    register repv env = rep_special_bindings;
-    while (env != Qnil && rep_CAAR(env) != sym)
-	env = rep_CDR(env);
-    return env != Qnil ? rep_CAR(env) : env;
+    register repv env;
+    for (env = rep_special_bindings; env != Qnil; env = rep_CDR (env))
+    {
+	if (rep_CAAR(env) == sym)
+	    return rep_CAR (env);
+    }
+    return Qnil;
 }
 
 static repv
@@ -552,7 +513,7 @@ search_special_bindings (repv sym)
 static inline int
 inlined_search_special_environment (repv sym)
 {
-    register repv env = rep_CDR(rep_special_env);
+    register repv env = rep_SPECIAL_ENV;
     while (rep_CONSP(env) && rep_CAR(env) != sym)
 	env = rep_CDR(env);
 
@@ -565,11 +526,36 @@ inlined_search_special_environment (repv sym)
 }
 
 static int
-search_special_environment (repv sym)
+search_special_environment__ (repv sym)
 {
     return inlined_search_special_environment (sym);
 }
 
+static inline int
+search_special_environment (repv sym)
+{
+    if (rep_SPECIAL_ENV == Qt)
+	return -1;
+    else
+	return search_special_environment__ (sym);
+}
+
+repv
+rep_call_with_closure (repv closure, repv (*fun)(repv arg), repv arg)
+{
+    repv ret = rep_NULL;
+    if (rep_FUNARGP (closure))
+    {
+	struct rep_Call lc;
+	lc.fun = lc.args = lc.args_evalled_p = Qnil;
+	rep_PUSH_CALL (lc);
+	rep_USE_FUNARG (closure);
+	ret = fun (arg);
+	rep_POP_CALL (lc);
+    }
+    return ret;
+}
+
 
 /* Symbol binding */
 
@@ -580,7 +566,7 @@ repv
 rep_bind_symbol(repv oldList, repv symbol, repv newVal)
 {
     if (oldList == Qnil)
-	oldList = Fcons (rep_MAKE_INT(0), rep_MAKE_INT(0));
+	oldList = rep_NEW_FRAME;
 
     if (rep_SYM(symbol)->car & rep_SF_SPECIAL)
     {
@@ -589,16 +575,15 @@ rep_bind_symbol(repv oldList, repv symbol, repv newVal)
 	{
 	    rep_special_bindings = Fcons (Fcons (symbol, newVal),
 					  rep_special_bindings);
-	    rep_CAR(oldList) = rep_MAKE_INT(rep_INT(rep_CAR(oldList)) + 1);
+	    oldList = rep_MARK_SPEC_BINDING (oldList);
 	}
 	else
 	    Fsignal (Qvoid_value, rep_LIST_1(symbol));
     }
     else
     {
-	/* lexical binding (also in lispmach.c:OP_BIND) */
-	rep_env = Fcons (Fcons (symbol, newVal), rep_env);
-	rep_CDR(oldList) = rep_MAKE_INT(rep_INT(rep_CDR(oldList)) + 1);
+	rep_env = Fcons (Fcons (LEXTAG, Fcons (symbol, newVal)), rep_env);
+	oldList = rep_MARK_LEX_BINDING (oldList);
     }
     return oldList;
 }
@@ -611,35 +596,45 @@ rep_unbind_symbols(repv oldList)
     if (oldList != Qnil)
     {
 	register repv tem;
+	int lexicals, specials;
 	int i;
 
-	assert (rep_CONSP(oldList));
-	assert (rep_INTP(rep_CAR(oldList)));
-	assert (rep_INTP(rep_CDR(oldList)));
+	assert (rep_INTP(oldList));
+
+	lexicals = rep_LEX_BINDINGS (oldList);
+	specials = rep_SPEC_BINDINGS (oldList);
 
 	tem = rep_env;
-	for (i = rep_INT (rep_CDR (oldList)); i > 0; i--)
+	for (i = lexicals; i > 0; i--)
 	    tem = rep_CDR (tem);
 	rep_env = tem;
 
 	tem = rep_special_bindings;
-	for (i = rep_INT (rep_CAR (oldList)); i > 0; i--)
+	for (i = specials; i > 0; i--)
+	{
 	    tem = rep_CDR (tem);
+	}
 	rep_special_bindings = tem;
 
 	assert (rep_special_bindings != rep_void_value);
 	assert (rep_env != rep_void_value);
 
-	return rep_INT (rep_CAR (oldList));
+	return specials;
     }
     else
 	return 0;
 }
 
+repv
+rep_add_binding_to_env (repv env, repv sym, repv value)
+{
+    return Fcons (Fcons (LEXTAG, Fcons (sym, value)), env);
+}
+
 
 /* More lisp functions */
 
-DEFUN("defvar", Fdefvar, Sdefvar, (repv args), rep_SF) /*
+DEFUN("defvar", Fdefvar, Sdefvar, (repv args, repv tail_posn), rep_SF) /*
 ::doc:defvar::
 defvar NAME DEFAULT-VALUE [DOC-STRING]
 
@@ -667,11 +662,16 @@ variable will be set (if necessary) not the local value.
 	if(!rep_NILP(tmp))
 	{
 	    /* Variable is bound, see if it's an autoload defn to overwrite. */
-	    repv val = rep_SYM(sym)->value;
+	    repv val = Fsymbol_value (sym, Qt);
 	    if (rep_FUNARGP(val))
+	    {
 		val = rep_FUNARG(val)->fun;
-	    if(rep_CONSP(val) && rep_CAR(val) == Qautoload)
-		tmp = Qnil;
+		if(rep_CONSP(val) && rep_CAR(val) == Qautoload)
+		{
+		    Fmakunbound (sym);
+		    tmp = Qnil;
+		}
+	    }
 	}
 
 	/* Only allowed to defvar in restricted environments
@@ -683,14 +683,26 @@ variable will be set (if necessary) not the local value.
 	    return Fsignal (Qvoid_value, rep_LIST_1(sym));	/* XXX */
 	}
 
+	/* if initially making it special, check for a lexical binding
+	   in the current module */
+	if (!(rep_SYM(sym)->car & rep_SF_SPECIAL))
+	{
+	    repv tem = rep_get_initial_special_value (sym);
+	    if (tem)
+	    {
+		val = tem;
+		tmp = Qnil;
+	    }
+	}
+
 	/* Only set the [default] value if its not boundp or
 	   the definition is weak and we're currently unrestricted */
 	if(rep_NILP(tmp)
 	   || ((rep_SYM(sym)->car & rep_SF_WEAK)
 	       && !(rep_SYM(sym)->car & rep_SF_WEAK_MOD)
-	       && rep_CDR(rep_special_env) == Qt))
+	       && rep_SPECIAL_ENV == Qt))
 	{
-	    rep_SYM(sym)->value = val;
+	    F_structure_set (rep_specials_structure, sym, val);
 	}
 
 	rep_SYM(sym)->car |= rep_SF_SPECIAL | rep_SF_DEFVAR;
@@ -701,10 +713,9 @@ variable will be set (if necessary) not the local value.
 	       environment sets it as weak, and adds it to the env */
 
 	    rep_SYM(sym)->car |= rep_SF_WEAK;
-	    rep_CDR(rep_special_env) = Fcons (sym, rep_CDR(rep_special_env));
+	    rep_SPECIAL_ENV = Fcons (sym, rep_SPECIAL_ENV);
 	}
-	else if (rep_CDR(rep_special_env) == Qt
-		 && (rep_SYM(sym)->car & rep_SF_WEAK))
+	else if (rep_SPECIAL_ENV == Qt && (rep_SYM(sym)->car & rep_SF_WEAK))
 	{
 	    /* defvar'ing a weak variable from an unrestricted
 	       environment removes the weak status, but marks
@@ -761,7 +772,7 @@ values look for one of those first.
 	int spec = inlined_search_special_environment (sym);
 	/* modified-weak specials can only be accessed from an
 	   unrestricted environment */
-	if (spec < 0 || !(rep_SYM(sym)->car & rep_SF_WEAK_MOD))
+	if (spec < 0 || (spec > 0 && !(rep_SYM(sym)->car & rep_SF_WEAK_MOD)))
 	{
 	    if(rep_SYM(sym)->car & rep_SF_LOCAL)
 		val = (*rep_deref_local_symbol_fun)(sym);
@@ -771,7 +782,7 @@ values look for one of those first.
 		if (tem != Qnil)
 		    val = rep_CDR (tem);
 		else
-		    val = rep_SYM(sym)->value;
+		    val = F_structure_ref (rep_specials_structure, sym);
 	    }
 	}
     }
@@ -779,10 +790,10 @@ values look for one of those first.
     {
 	/* lexical variable */
 	repv tem = search_environment (sym);
-	if (rep_CONSP(tem))
+	if (tem != Qnil)
 	    val = rep_CDR(tem);
-	else if (tem == Qt)
-	    val = rep_SYM(sym)->value;
+	else
+	    val = F_structure_ref (rep_structure, sym);
     }
 
     if (rep_SYM(sym)->car & rep_SF_DEBUG)
@@ -804,18 +815,22 @@ SYMBOL in buffers or windows which do not have their own local value.
 ::end:: */
 {
     repv val = rep_void_value;
-    int spec;
     rep_DECLARE1(sym, rep_SYMBOLP);
-    
-    spec = search_special_environment (sym);
-    if (spec < 0 || !(rep_SYM(sym)->car & rep_SF_WEAK_MOD))
+
+    if (rep_SYM(sym)->car & rep_SF_SPECIAL)
     {
-	repv tem = search_special_bindings (sym);
-	if (tem != Qnil)
-	    val = rep_CDR (tem);
-	else
-	    val = rep_SYM(sym)->value;
+	int spec = search_special_environment (sym);
+	if (spec < 0 || (spec > 0 && !(rep_SYM(sym)->car & rep_SF_WEAK_MOD)))
+	{
+	    repv tem = search_special_bindings (sym);
+	    if (tem != Qnil)
+		val = rep_CDR (tem);
+	    else
+		val = F_structure_ref (rep_specials_structure, sym);
+	}
     }
+    else
+	val = F_structure_ref (rep_structure, sym);
 
     if(no_err == Qnil && rep_VOIDP(val))
 	return Fsignal(Qvoid_value, rep_LIST_1(sym));
@@ -837,9 +852,6 @@ SYMBOL the buffer-local value in the current buffer is set. Returns repv.
        instruction in lispmach.c */
     rep_DECLARE1(sym, rep_SYMBOLP);
 
-    if (rep_SYM(sym)->car & rep_SF_CONSTANT)
-	return Fsignal(Qsetting_constant, rep_LIST_1(sym));
-
     if (rep_SYM(sym)->car & rep_SF_SPECIAL)
     {
 	int spec = inlined_search_special_environment (sym);
@@ -863,49 +875,23 @@ SYMBOL the buffer-local value in the current buffer is set. Returns repv.
 	    if (tem != Qnil)
 		rep_CDR (tem) = val;
 	    else
-		rep_SYM(sym)->value = val;
+		val = F_structure_set (rep_specials_structure, sym, val);
 	}
 	else
-	    return Fsignal (Qvoid_value, rep_LIST_1(sym));	/* XXX */
+	    val = Fsignal (Qvoid_value, rep_LIST_1(sym));	/* XXX */
     }
     else
     {
 	/* lexical binding */
 	repv tem = search_environment (sym);
-	if (rep_CONSP(tem))
+	if (tem != Qnil)
 	    rep_CDR(tem) = val;
-	else if (tem == Qt)
-	    rep_SYM(sym)->value = val;
 	else
-	    return Fsignal (Qvoid_value, rep_LIST_1(sym));	/* XXX */
+	    val = F_structure_set (rep_structure, sym, val);
     }
     return val;
 }
 
-DEFUN("mark-symbol-defined", Fmark_symbol_defined,
-      Smark_symbol_defined, (repv sym), rep_Subr1) /*
-::doc:mark-symbol-defined::
-mark-symbol-defined SYMBOL
-::end:: */
-{
-    int spec;
-    rep_DECLARE1(sym, rep_SYMBOLP);
-    spec = search_special_environment (sym);
-    if (spec)
-    {
-	rep_SYM(sym)->car |= rep_SF_DEFVAR;
-	if (rep_CDR(rep_special_env) == Qt
-	    && (rep_SYM(sym)->car & rep_SF_WEAK))
-	{
-	    rep_SYM(sym)->car &= ~rep_SF_WEAK;
-	    rep_SYM(sym)->car |= rep_SF_WEAK_MOD;
-	}
-	return sym;
-    }
-    else
-	return Fsignal (Qvoid_value, rep_LIST_1(sym));	/* XXX */
-}
-
 DEFUN("set-default", Fset_default, Sset_default,
       (repv sym, repv val), rep_Subr2) /*
 ::doc:set-default::
@@ -914,30 +900,28 @@ set-default SYMBOL VALUE
 Sets the default value of SYMBOL to VALUE, then returns VALUE.
 ::end:: */
 {
-    int spec;
-
     rep_DECLARE1(sym, rep_SYMBOLP);
-
-    if (rep_SYM(sym)->car & rep_SF_CONSTANT)
-	return Fsignal(Qsetting_constant, rep_LIST_1(sym));
-
-    spec = search_special_environment (sym);
-    if (spec)
+    if (rep_SYM (sym)->car & rep_SF_SPECIAL)
     {
-	repv tem;
+	int spec = search_special_environment (sym);
+	if (spec)
+	{
+	    repv tem;
 
-	if (spec > 0 && rep_SYM(sym)->car & rep_SF_WEAK_MOD)
-	    return Fsignal (Qvoid_value, rep_LIST_1(sym));	/* XXX */
+	    if (spec > 0 && rep_SYM(sym)->car & rep_SF_WEAK_MOD)
+		return Fsignal (Qvoid_value, rep_LIST_1(sym));	/* XXX */
 
-	tem = search_special_bindings (sym);
-	if (tem != Qnil)
-	    rep_CDR (tem) = val;
+	    tem = search_special_bindings (sym);
+	    if (tem != Qnil)
+		rep_CDR (tem) = val;
+	    else
+		val = F_structure_set (rep_specials_structure, sym, val);
+	}
 	else
-	    rep_SYM(sym)->value = val;
-	rep_SYM(sym)->car |= rep_SF_SPECIAL;
+	    return Fsignal (Qvoid_value, rep_LIST_1(sym));	/* XXX */
     }
     else
-	return Fsignal (Qvoid_value, rep_LIST_1(sym));	/* XXX */
+	F_structure_set (rep_structure, sym, val);
     return val;
 }
 
@@ -954,7 +938,7 @@ Sets the property list of SYMBOL to PROP-LIST, returns PROP-LIST.
     if (spec == 0)
 	return Fsignal (Qvoid_value, rep_LIST_1(sym));	/* XXX */
 
-    rep_SYM(sym)->prop_list = prop;
+    F_structure_set (plist_structure, sym, prop);
     return prop;
 }
 
@@ -976,13 +960,20 @@ default-boundp SYMBOL
 Returns t if SYMBOL has a default value.
 ::end:: */
 {
-    repv tem;
     rep_DECLARE1(sym, rep_SYMBOLP);
-    tem = search_special_bindings (sym);
-    if (tem != Qnil)
-	return rep_VOIDP (rep_CDR (tem));
+    if (rep_SYM(sym)->car & rep_SF_SPECIAL)
+    {
+	repv tem = search_special_bindings (sym);
+	if (tem != Qnil)
+	    return rep_VOIDP (rep_CDR (tem));
+	else
+	{
+	    tem = F_structure_ref (rep_specials_structure, sym);
+	    return rep_VOIDP (tem) ? Qnil : Qt;
+	}
+    }
     else
-	return (rep_VOIDP (rep_SYM (sym)->value)) ? Qnil : Qt;
+	return F_structure_bound_p (rep_structure, sym);
 }
 
 DEFUN("boundp", Fboundp, Sboundp, (repv sym), rep_Subr1) /*
@@ -1004,12 +995,14 @@ Returns the property-list of SYMBOL.
 ::end:: */
 {
     int spec;
+    repv plist;
     rep_DECLARE1(sym, rep_SYMBOLP);
     spec = search_special_environment (sym);
     if (spec == 0)
 	return Fsignal (Qvoid_value, rep_LIST_1(sym));	/* XXX */
 
-    return rep_SYM(sym)->prop_list;
+    plist = F_structure_ref (plist_structure, sym);
+    return rep_VOIDP (plist) ? Qnil : plist;
 }
 
 DEFUN("gensym", Fgensym, Sgensym, (void), rep_Subr0) /*
@@ -1040,7 +1033,7 @@ Returns t if ARG is a symbol.
     return(rep_SYMBOLP(sym) ? Qt : Qnil);
 }
 
-DEFUN("setq", Fsetq, Ssetq, (repv args), rep_SF) /*
+DEFUN("setq", Fsetq, Ssetq, (repv args, repv tail_posn), rep_SF) /*
 ::doc:setq::
 setq { SYMBOL FORM }...
 
@@ -1092,7 +1085,9 @@ Returns the value of SYMBOL's property PROPERTY. See `put'.
 {
     repv plist;
     rep_DECLARE1(sym, rep_SYMBOLP);
-    plist = rep_SYM(sym)->prop_list;
+    plist = F_structure_ref (plist_structure, sym);
+    if (rep_VOIDP (plist))
+	return Qnil;
     while(rep_CONSP(plist) && rep_CONSP(rep_CDR(plist)))
     {
 	if(rep_CAR(plist) == prop
@@ -1114,14 +1109,17 @@ Sets the value of SYMBOL's property PROPERTY to repv, this value can be
 retrieved with the `get' function.
 ::end:: */
 {
-    repv plist;
+    repv plist, old;
     int spec;
     rep_DECLARE1(sym, rep_SYMBOLP);
     spec = search_special_environment (sym);
     if (spec == 0)
 	return Fsignal (Qvoid_value, rep_LIST_1(sym));	/* XXX */
 
-    plist = rep_SYM(sym)->prop_list;
+    old = F_structure_ref (plist_structure, sym);
+    if (rep_VOIDP (old))
+	old = Qnil;
+    plist = old;
     while(rep_CONSP(plist) && rep_CONSP(rep_CDR(plist)))
     {
 	if(rep_CAR(plist) == prop
@@ -1139,13 +1137,8 @@ retrieved with the `get' function.
 	}
 	plist = rep_CDR(rep_CDR(plist));
     }
-    plist = Fcons(prop, Fcons(val, rep_SYM(sym)->prop_list));
-    if(plist)
-    {
-	rep_SYM(sym)->prop_list = plist;
-	return val;
-    }
-    return rep_NULL;
+    F_structure_set (plist_structure, sym, Fcons (prop, Fcons (val, old)));
+    return val;
 }
 
 DEFUN("apropos", Fapropos, Sapropos, (repv re, repv pred, repv ob), rep_Subr3) /*
@@ -1200,34 +1193,6 @@ next:
     return rep_NULL;
 }
 
-DEFUN("set-const-variable", Fset_const_variable, Sset_const_variable, (repv sym, repv stat), rep_Subr2) /*
-::doc:set-const-variable::
-set-const-variable SYMBOL
-
-Flags that the value of SYMBOL may not be changed.
-::end:: */
-{
-    rep_DECLARE1(sym, rep_SYMBOLP);
-    if(rep_NILP(stat))
-	rep_SYM(sym)->car |= rep_SF_CONSTANT;
-    else
-	rep_SYM(sym)->car &= ~rep_SF_CONSTANT;
-    return(sym);
-}
-
-DEFUN("const-variable-p", Fconst_variable_p, Sconst_variable_p, (repv sym), rep_Subr1) /*
-::doc:const-variable-p::
-const-variable-p SYMBOL
-
-Return t is `set-const-variable' has been called on SYMBOL.
-::end:: */
-{
-    rep_DECLARE1(sym, rep_SYMBOLP);
-    if(rep_SYM(sym)->car & rep_SF_CONSTANT)
-	return(Qt);
-    return(Qnil);
-}
-
 DEFUN("make-variable-special", Fmake_variable_special,
       Smake_variable_special, (repv sym), rep_Subr1)
 {
@@ -1236,6 +1201,12 @@ DEFUN("make-variable-special", Fmake_variable_special,
     spec = search_special_environment (sym);
     if (spec == 0)
 	return Fsignal (Qvoid_value, rep_LIST_1(sym));	/* XXX */
+    if (!(rep_SYM(sym)->car & rep_SF_SPECIAL))
+    {
+	repv tem = rep_get_initial_special_value (sym);
+	if (tem)
+	    F_structure_set (rep_specials_structure, sym, tem);
+    }
     rep_SYM(sym)->car |= rep_SF_SPECIAL;
     return sym;
 }
@@ -1315,20 +1286,23 @@ rep_symbols_init(void)
        all fields in case it was dumped. */
     Qnil = Fintern(rep_VAL(&str_nil), rep_obarray);
     rep_mark_static(&Qnil);
-    rep_SYM(Qnil)->value = Qnil;
-    rep_SYM(Qnil)->prop_list = Qnil;
-    rep_SYM(Qnil)->car |= rep_SF_CONSTANT | rep_SF_SPECIAL;
+    rep_INTERN(t);
 
-    rep_INTERN_SPECIAL(t);
-    rep_SYM(Qt)->value = Qt;
-    rep_SYM(Qt)->car |= rep_SF_CONSTANT;
+    rep_pre_structures_init ();
 
     rep_USE_DEFAULT_ENV;
     rep_special_bindings = Qnil;
     rep_mark_static (&rep_env);
-    rep_mark_static (&rep_special_env);
     rep_mark_static (&rep_special_bindings);
 
+    F_structure_set (rep_structure, Qnil, Qnil);
+    F_structure_set (rep_structure, Qt, Qt);
+    Fmake_binding_immutable (Qnil);
+    Fmake_binding_immutable (Qt);
+
+    plist_structure = F_make_structure (Qnil, Qnil, Qnil, Qnil);
+    rep_mark_static (&plist_structure);
+
     rep_INTERN(documentation);
     rep_INTERN(permanent_local);
     rep_ADD_SUBR(Smake_symbol);
@@ -1340,14 +1314,13 @@ rep_symbols_init(void)
     rep_ADD_SUBR(Smake_closure);
     rep_ADD_SUBR(Sclosure_function);
     rep_ADD_SUBR(Sset_closure_function);
+    rep_ADD_SUBR(Sclosure_structure);
+    rep_ADD_SUBR(Sclosure_name);
     rep_ADD_SUBR(Sclosurep);
-    rep_ADD_SUBR(Ssave_environment);
-    rep_ADD_SUBR(Sset_environment);
     rep_ADD_SUBR(Sset_special_environment);
     rep_ADD_SUBR(Sdefvar);
     rep_ADD_SUBR(Ssymbol_value);
     rep_ADD_SUBR_INT(Sset);
-    rep_ADD_SUBR(Smark_symbol_defined);
     rep_ADD_SUBR(Ssetplist);
     rep_ADD_SUBR(Ssymbol_name);
     rep_ADD_SUBR(Sdefault_value);
@@ -1362,24 +1335,9 @@ rep_symbols_init(void)
     rep_ADD_SUBR(Sget);
     rep_ADD_SUBR(Sput);
     rep_ADD_SUBR(Sapropos);
-    rep_ADD_SUBR(Sset_const_variable);
-    rep_ADD_SUBR(Sconst_variable_p);
     rep_ADD_SUBR(Smake_variable_special);
     rep_ADD_SUBR(Sspecial_variable_p);
     rep_ADD_SUBR_INT(Strace);
     rep_ADD_SUBR_INT(Suntrace);
     rep_ADD_SUBR(Sobarray);
 }
-
-void
-rep_symbols_kill(void)
-{
-    rep_symbol_block *sb = symbol_block_chain;
-    while(sb)
-    {
-	rep_symbol_block *nxt = sb->next;
-	rep_FREE_CELL(sb);
-	sb = nxt;
-    }
-    symbol_block_chain = NULL;
-}
