@@ -1,6 +1,6 @@
 /* timers.c -- call a function after a period of time has passed
    Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
-   $Id: timers.c,v 1.9 2000/02/28 08:29:28 john Exp $
+   $Id: timers.c,v 1.10 2000/04/07 16:31:35 john Exp $
 
    This file is part of librep.
 
@@ -80,8 +80,6 @@ static int pipe_fds[2];
 /* Contains SIGALRM */
 static sigset_t alrm_sigset;
 
-DEFSYM(timers, "timers");
-
 
 
 static RETSIGTYPE
@@ -358,6 +356,7 @@ timer_print (repv stream, repv arg)
 repv
 rep_dl_init (void)
 {
+    repv tem;
     timer_type = rep_register_new_type ("timer", 0, timer_print, timer_print,
 					timer_sweep, timer_mark,
 					timer_mark_active, 0, 0, 0, 0, 0, 0);
@@ -370,12 +369,11 @@ rep_dl_init (void)
     sigaddset (&alrm_sigset, SIGALRM);
     rep_sig_restart (SIGALRM, rep_TRUE);
 
+    tem = rep_push_structure ("timers");
     rep_ADD_SUBR(Smake_timer);
     rep_ADD_SUBR(Sdelete_timer);
     rep_ADD_SUBR(Sset_timer);
-
-    rep_INTERN (timers);
-    return Qtimers;
+    return rep_pop_structure (tem);
 }
 
 void
