@@ -1,6 +1,6 @@
 /* unix_dl.c -- Dynamic loading of C modules
    Copyright (C) 1998 John Harper <john@dcs.warwick.ac.uk>
-   $Id: unix_dl.c,v 1.26 2000/03/21 17:28:47 john Exp $
+   $Id: unix_dl.c,v 1.33 2000/07/09 11:13:42 john Exp $
 
    This file is part of Jade.
 
@@ -113,6 +113,7 @@ struct dl_lib_info {
     struct dl_lib_info *next;
     repv file_name;
     repv feature_sym;
+    repv structure;
     void *handle;
 };
 
@@ -191,12 +192,9 @@ load_requires (char *ptr)
     while (*ptr != 0)
     {
 	char *end = ptr + strcspn (ptr, " \t");
-	repv name = rep_string_dupn (ptr, end - ptr);
-	if (find_dl_by_feature (name) == 0)
-	{
-	    if (Fload (name, Qnil, Qnil, Qnil, Qnil) == rep_NULL)
-		return rep_FALSE;
-	}
+	repv sym = Fintern (rep_string_dupn (ptr, end - ptr), Qnil);
+	if (F_intern_structure (sym) == rep_NULL)
+	    return rep_FALSE;
 	ptr = end + strspn (end, " \t");
     }
     return rep_TRUE;
@@ -211,7 +209,7 @@ signal_error (char *msg)
 	fprintf (stderr, "error: %s\n", msg);
 }
 
-void *
+repv
 rep_open_dl_library(repv file_name)
 {
     struct dl_lib_info *x = find_dl(file_name);
@@ -237,7 +235,7 @@ rep_open_dl_library(repv file_name)
 		char *ptr = buf + sizeof("dlname='") - 1;
 		u_char *base;
 		char *end = strchr(ptr, '\'');
-		if (end != 0)
+		if (end != 0 && end > ptr)
 		{
 		    *end = 0;
 		    base = strrchr(rep_STR(file_name), '/');
@@ -332,6 +330,7 @@ rep_open_dl_library(repv file_name)
 	    x->file_name = file_name;
 	    x->handle = handle;
 	    x->feature_sym = Qnil;
+	    x->structure = Qnil;
 
 	    x->next = dl_list;
 	    dl_list = x;
@@ -357,6 +356,13 @@ rep_open_dl_library(repv file_name)
 		}
 		else if (ret && rep_SYMBOLP(ret) && ret != Qt)
 		    x->feature_sym = ret;
+		else if (ret && rep_STRUCTUREP (ret))
+		{
+		    x->structure = ret;
+		    ret = F_structure_name (ret);
+		    if (ret && rep_SYMBOLP (ret))
+			x->feature_sym = ret;
+		}
 	    }
 
 	    feature_sym = x_dlsym (handle, "rep_dl_feature");
@@ -371,17 +377,26 @@ rep_open_dl_library(repv file_name)
 	    {
 		while(*functions != 0)
 		{
-		    rep_add_subr(*functions);
+		    rep_add_subr(*functions, rep_TRUE);
 		    functions++;
 		}
 	    }
-
-	    if (x->feature_sym != Qnil)
-		Fprovide (x->feature_sym);
 	}
     }
 out:
-    return x;
+
+    if (x != 0)
+    {
+	if (x->feature_sym != Qnil && x->structure == Qnil)
+	{
+	    /* only `provide' the feature if there's no associated
+	       structure (since we haven't actually imported it) */
+	    Fprovide (x->feature_sym);
+	}
+	return x->structure;
+    }
+    else
+	return rep_NULL;
 }
 
 void
@@ -392,6 +407,7 @@ rep_mark_dl_data(void)
     {
 	rep_MARKVAL(x->file_name);
 	rep_MARKVAL(x->feature_sym);
+	rep_MARKVAL(x->structure);
 	x = x->next;
     }
 }
