@@ -1,6 +1,6 @@
 /* unix_main.c -- Miscellaneous functions for Unix
    Copyright (C) 1993, 1994 John Harper <john@dcs.warwick.ac.uk>
-   $Id: unix_main.c,v 1.57 2000/02/08 14:16:10 john Exp $
+   $Id: unix_main.c,v 1.59 2000/07/13 10:44:36 john Exp $
 
    This file is part of Jade.
 
@@ -87,6 +87,20 @@ rep_time(void)
     return time(0);
 }
 
+rep_long_long
+rep_utime (void)
+{
+    rep_long_long t;
+#ifdef HAVE_GETTIMEOFDAY
+    struct timeval time;
+    gettimeofday (&time, 0);
+    t = ((rep_long_long) time.tv_sec * 1000000) + time.tv_usec;
+#else
+    t = (rep_long_long) rep_time () * 1000000;
+#endif
+    return t;
+}
+
 void
 rep_sleep_for(long secs, long msecs)
 {
@@ -858,9 +872,12 @@ rep_sys_os_init(void)
 
     /* Initialise process-environment variable */
     env = Qnil;
-    ptr = environ;
-    while(*ptr != 0)
-	env = Fcons(rep_string_dup(*ptr++), env);
+    if (environ != 0)
+    {
+	ptr = environ;
+	while(*ptr != 0)
+	    env = Fcons(rep_string_dup(*ptr++), env);
+    }
     Fset (Qprocess_environment, env);
 
 #ifdef DEBUG_SYS_ALLOC
