@@ -1,6 +1,6 @@
 /* unix_processes.c -- Subprocess handling for Unix
    Copyright (C) 1993, 1994 John Harper <john@dcs.warwick.ac.uk>
-   $Id: unix_processes.c,v 1.67 1999/11/29 18:37:58 john Exp $
+   $Id: unix_processes.c,v 1.69 2000/07/10 21:12:44 john Exp $
 
    This file is part of Jade.
 
@@ -109,8 +109,8 @@ struct Proc
 };
 
 /* Status is two bits above the type code (presently 8->9) */
-#define PR_ACTIVE  (1 << (rep_CELL8_TYPE_BITS + 0))	/* active, may be stopped */
-#define PR_STOPPED (2 << (rep_CELL8_TYPE_BITS + 1))	/* stopped */
+#define PR_ACTIVE  (1 << (rep_CELL16_TYPE_BITS + 0))	/* active, may be stopped */
+#define PR_STOPPED (2 << (rep_CELL16_TYPE_BITS + 1))	/* stopped */
 #define PR_DEAD    0
 #define PR_RUNNING PR_ACTIVE
 
@@ -137,7 +137,7 @@ DEFSYM(socketpair, "socketpair");
     ((p)->pr_ConnType == Qsocketpair)
 
 #define VPROC(v)	((struct Proc *)rep_PTR(v))
-#define PROCESSP(v)	rep_CELL8_TYPEP(v, rep_Process)
+#define PROCESSP(v)	rep_CELL16_TYPEP(v, process_type)
 
 /* Handy debugging macro */
 #if 0
@@ -150,6 +150,8 @@ static struct Proc *process_chain;
 static struct Proc *notify_chain;
 static int process_run_count;
 
+static int process_type;
+
 /* Set to rep_TRUE by the SIGCHLD handler */
 static volatile rep_bool got_sigchld;
 
@@ -1037,7 +1039,7 @@ If the DIR parameter is nil it will be inherited from the
     {
 	rep_GC_root gc_pr;
 	rep_data_after_gc += sizeof (struct Proc);
-	VPROC(pr)->pr_Car = rep_Process;
+	VPROC(pr)->pr_Car = process_type;
 	VPROC(pr)->pr_Next = process_chain;
 	process_chain = VPROC(pr);
 	VPROC(pr)->pr_NotifyNext = NULL;
@@ -1877,9 +1879,10 @@ rep_system (char *command)
     switch (pid)
     {
 	char *argv[4];
+	DEFSTRING (cant_fork, "can't fork ()");
 
     case -1:
-	return Fsignal (Qerror, Fcons (rep_string_dup ("Can't fork"), Qnil));
+	return Fsignal (Qerror, Fcons (rep_VAL (&cant_fork), Qnil));
 
     case 0:
 	child_build_environ ();
@@ -1916,7 +1919,10 @@ rep_system (char *command)
 		break;
 	    case -1:
 		if (errno != EINTR && errno != EAGAIN)
-		    return Fsignal (Qerror, Fcons (rep_string_dup ("Can't waitpid"), Qnil));
+		{
+		    DEFSTRING (cant_waitpid, "can't waitpid ()");
+		    return Fsignal (Qerror, Fcons (rep_VAL (&cant_waitpid), Qnil));
+		}
 		break;
 	    default:
 		return rep_MAKE_INT (status);
@@ -1978,9 +1984,11 @@ rep_proc_init(void)
     rep_ADD_SUBR(Sactive_processes);
     rep_ADD_SUBR(Saccept_process_output);
 
-    rep_register_type(rep_Process, "subprocess", rep_ptr_cmp, proc_prin, proc_prin,
-		  proc_sweep, proc_mark, mark_active_processes,
-		  0, 0, proc_putc, proc_puts, 0, 0);
+    process_type = rep_register_new_type ("subprocess", rep_ptr_cmp,
+					  proc_prin, proc_prin,
+					  proc_sweep, proc_mark,
+					  mark_active_processes,
+					  0, 0, proc_putc, proc_puts, 0, 0);
 }
 
 void
