@@ -1,6 +1,6 @@
 /* values.c -- Handling of Lisp data (includes garbage collection)
    Copyright (C) 1993, 1994 John Harper <john@dcs.warwick.ac.uk>
-   $Id: values.c,v 1.52 2000/04/06 21:47:38 john Exp $
+   $Id: values.c,v 1.62 2000/07/14 11:08:51 john Exp $
 
    This file is part of Jade.
 
@@ -49,22 +49,8 @@ char *alloca ();
 
 /* #define GC_MONITOR_STK */
 
-/* #define DEBUG_GC */
-
-#define rep_CONSBLK_SIZE	510		/* ~4k */
 #define rep_STRINGBLK_SIZE	510		/* ~4k */
 
-/* Structure of cons allocation blocks */
-typedef struct rep_cons_block_struct {
-    union {
-	struct rep_cons_block_struct *p;
-	/* ensure that the following cons cell is aligned to at
-	   least sizeof (rep_cons) (for the dcache) */
-	rep_cons dummy;
-    } next;
-    rep_cons cons[rep_CONSBLK_SIZE];
-} rep_cons_block;
-
 /* Structure of string header allocation blocks */
 typedef struct rep_string_block_struct {
     union {
@@ -76,10 +62,6 @@ typedef struct rep_string_block_struct {
     rep_string data[rep_STRINGBLK_SIZE];
 } rep_string_block;
 
-#ifdef DEBUG_GC
-static int debug_gc = 0;
-#endif
-
 /* Dumped data */
 rep_cons *rep_dumped_cons_start, *rep_dumped_cons_end;
 rep_symbol *rep_dumped_symbols_start, *rep_dumped_symbols_end;
@@ -116,7 +98,7 @@ rep_register_type(u_int code, char *name,
     rep_type *t = rep_alloc(sizeof(rep_type));
     if (t == 0)
     {
-	rep_mem_error();
+	rep_mem_error ();
 	return;
     }
     t->code = code;
@@ -270,7 +252,7 @@ rep_box_string (char *ptr, long len)
 	    string_freelist = cb->data;
 	}
 	else
-	    return rep_mem_error();
+	    return rep_mem_error ();
 	str = string_freelist;
     }
     string_freelist = rep_STRING(str->car);
@@ -473,9 +455,36 @@ rep_unbox_pointer (repv v)
 
 /* Cons */
 
-static rep_cons_block *cons_block_chain;
-static rep_cons *cons_freelist;
-static int allocated_cons, used_cons;
+rep_cons_block *rep_cons_block_chain;
+rep_cons *rep_cons_freelist;
+int rep_allocated_cons, rep_used_cons;
+
+rep_cons *
+rep_allocate_cons (void)
+{
+    rep_cons *cn;
+    cn = rep_cons_freelist;
+    if(cn == NULL)
+    {
+	rep_cons_block *cb;
+	cb = rep_alloc(sizeof(rep_cons_block));
+	if(cb != NULL)
+	{
+	    int i;
+	    rep_allocated_cons += rep_CONSBLK_SIZE;
+	    cb->next.p = rep_cons_block_chain;
+	    rep_cons_block_chain = cb;
+	    for(i = 0; i < (rep_CONSBLK_SIZE - 1); i++)
+		cb->cons[i].cdr = rep_CONS_VAL(&cb->cons[i + 1]);
+	    cb->cons[i].cdr = 0;
+	    rep_cons_freelist = cb->cons;
+	}
+	else
+	    return rep_CONS (rep_mem_error ());
+	cn = rep_cons_freelist;
+    }
+    return cn;
+}
 
 DEFUN("cons", Fcons, Scons, (repv car, repv cdr), rep_Subr2) /*
 ::doc:cons::
@@ -484,67 +493,53 @@ cons CAR CDR
 Returns a new cons-cell with car CAR and cdr CDR.
 ::end:: */
 {
-    rep_cons *cn;
-    cn = cons_freelist;
-    if(cn == NULL)
-    {
-	rep_cons_block *cb;
-	cb = rep_alloc(sizeof(rep_cons_block));
-	if(cb != NULL)
-	{
-	    int i;
-	    allocated_cons += rep_CONSBLK_SIZE;
-	    cb->next.p = cons_block_chain;
-	    cons_block_chain = cb;
-	    for(i = 0; i < (rep_CONSBLK_SIZE - 1); i++)
-		cb->cons[i].cdr = rep_CONS_VAL(&cb->cons[i + 1]);
-	    cb->cons[i].cdr = 0;
-	    cons_freelist = cb->cons;
-	}
-	else
-	    return rep_mem_error();
-	cn = cons_freelist;
-    }
-    cons_freelist = rep_CONS(cn->cdr);
-    cn->car = car;
-    cn->cdr = cdr;
-    used_cons++;
+    rep_cons *c = rep_cons_freelist;
+    if (c == 0)
+	c = rep_allocate_cons ();
+    rep_cons_freelist = rep_CONS (c->cdr);
+    rep_used_cons++;
     rep_data_after_gc += sizeof(rep_cons);
-    return rep_CONS_VAL(cn);
+
+    c->car = car;
+    c->cdr = cdr;
+    return rep_CONS_VAL (c);
 }
 
 void
 rep_cons_free(repv cn)
 {
-    rep_CDR(cn) = rep_CONS_VAL(cons_freelist);
-    cons_freelist = rep_CONS(cn);
-    used_cons--;
+    rep_CDR(cn) = rep_CONS_VAL(rep_cons_freelist);
+    rep_cons_freelist = rep_CONS(cn);
+    rep_used_cons--;
 }
 
 static void
 cons_sweep(void)
 {
     rep_cons_block *cb;
-    cons_freelist = 0;
-    used_cons = 0;
-    for (cb = cons_block_chain; cb != 0; cb = cb->next.p)
+    rep_cons *tem_freelist = 0;
+    int tem_used = 0;
+    for (cb = rep_cons_block_chain; cb != 0; cb = cb->next.p)
     {
-	int i;
-	rep_cons *this = cb->cons;
-	for (i = 0; i < rep_CONSBLK_SIZE; i++, this++)
+	register rep_cons *this = cb->cons;
+	rep_cons *last = cb->cons + rep_CONSBLK_SIZE;
+	while (this < last)
 	{
 	    if (!rep_GC_CONS_MARKEDP (rep_CONS_VAL (this)))
 	    {
-		this->cdr = rep_CONS_VAL (cons_freelist);
-		cons_freelist = rep_CONS (this);
+		this->cdr = rep_CONS_VAL (tem_freelist);
+		tem_freelist = rep_CONS (this);
 	    }
 	    else
 	    {
 		rep_GC_CLR_CONS (rep_CONS_VAL (this));
-		used_cons++;
+		tem_used++;
 	    }
+	    this++;
 	}
     }
+    rep_cons_freelist = tem_freelist;
+    rep_used_cons = tem_used;
 }
 
 static int
@@ -783,7 +778,7 @@ rep_bool rep_in_gc = rep_FALSE;
 /* rep_data_after_gc = bytes of storage used since last gc
    rep_gc_threshold = value that rep_data_after_gc should be before gc'ing
    rep_idle_gc_threshold = value that DAGC should be before gc'ing in idle time */
-int rep_data_after_gc, rep_gc_threshold = 100000, rep_idle_gc_threshold = 20000;
+int rep_data_after_gc, rep_gc_threshold = 200000, rep_idle_gc_threshold = 20000;
 
 #ifdef GC_MONITOR_STK
 static int *gc_stack_high_tide;
@@ -884,8 +879,6 @@ again:
 	/* Dumped symbols are dumped read-write, so no worries.. */
 	rep_GC_SET_CELL(val);
 	rep_MARKVAL(rep_SYM(val)->name);
-	rep_MARKVAL(rep_SYM(val)->value);
-	rep_MARKVAL(rep_SYM(val)->prop_list);
 	val = rep_SYM(val)->next;
 	if(val && !rep_INTP(val) && !rep_GC_MARKEDP(val))
 	    goto again;
@@ -907,8 +900,7 @@ again:
 	rep_GC_SET_CELL(val);
 	rep_MARKVAL(rep_FUNARG(val)->name);
 	rep_MARKVAL(rep_FUNARG(val)->env);
-	rep_MARKVAL(rep_FUNARG(val)->special_env);
-	rep_MARKVAL(rep_FUNARG(val)->fh_env);
+	rep_MARKVAL(rep_FUNARG(val)->structure);
 	val = rep_FUNARG(val)->fun;
 	if (val && !rep_GC_MARKEDP(val))
 	    goto again;
@@ -954,7 +946,7 @@ collection is triggered when the editor is idle.
     return rep_handle_var_int(val, &rep_idle_gc_threshold);
 }
 
-DEFUN_INT("garbage-collect", Fgarbage_collect, Sgarbage_collect, (repv noStats), rep_Subr1, "") /*
+DEFUN_INT("garbage-collect", Fgarbage_collect, Sgarbage_collect, (repv stats), rep_Subr1, "") /*
 ::doc:garbage-collect::
 garbage-collect
 
@@ -974,15 +966,6 @@ last garbage-collection is greater than `garbage-threshold'.
 
     rep_in_gc = rep_TRUE;
 
-#ifdef DEBUG_GC
-    if (debug_gc)
-    {
-	repv stream = Fstderr_file ();
-	rep_stream_puts (stream, "\n ** garbage collection:", -1, rep_FALSE);
-	Fbacktrace (stream);
-    }
-#endif
-
     rep_macros_before_gc ();
 
     /* mark static objects */
@@ -1026,8 +1009,7 @@ last garbage-collection is greater than `garbage-threshold'.
 	rep_MARKVAL(lc->fun);
 	rep_MARKVAL(lc->args);
 	rep_MARKVAL(lc->saved_env);
-	rep_MARKVAL(lc->saved_special_env);
-	rep_MARKVAL(lc->saved_fh_env);
+	rep_MARKVAL(lc->saved_structure);
 	/* don't bother marking `args_evalled_p' it's always `nil' or `t'  */
 	lc = lc->next;
     }
@@ -1037,6 +1019,7 @@ last garbage-collection is greater than `garbage-threshold'.
 
     /* Finished marking, start sweeping. */
 
+    rep_sweep_tuples ();
     for(i = 0; i < TYPE_HASH_SIZE; i++)
     {
 	rep_type *t = data_types[i];
@@ -1056,30 +1039,25 @@ last garbage-collection is greater than `garbage-threshold'.
 	    ((int)&dummy) - (int)gc_stack_high_tide);
 #endif
 
-#ifdef DEBUG_GC
-    if (debug_gc)
-    {
-	repv stream = Fstderr_file ();
-	rep_stream_putc (stream, '\n');
-    }
-#endif
-
     Fcall_hook (Qafter_gc_hook, Qnil, Qnil);
 
-    if(rep_NILP(noStats))
+    if(stats != Qnil)
     {
-	return(rep_list_4(Fcons(rep_MAKE_INT(used_cons),
-				rep_MAKE_INT(allocated_cons - used_cons)),
-		      Fcons(rep_MAKE_INT(rep_used_symbols),
-			    rep_MAKE_INT(rep_allocated_symbols
-					 - rep_used_symbols)),
-		      rep_list_3(rep_MAKE_INT(used_strings),
-				 rep_MAKE_INT(allocated_strings),
-				 rep_MAKE_INT(allocated_string_bytes)),
-		      rep_MAKE_INT(used_vector_slots)));
+	return rep_list_5(Fcons(rep_MAKE_INT(rep_used_cons),
+				rep_MAKE_INT(rep_allocated_cons - rep_used_cons)),
+			  Fcons(rep_MAKE_INT(rep_used_tuples),
+				rep_MAKE_INT(rep_allocated_tuples
+					     - rep_used_tuples)),
+			  rep_list_3(rep_MAKE_INT(used_strings),
+				     rep_MAKE_INT(allocated_strings),
+				     rep_MAKE_INT(allocated_string_bytes)),
+			  rep_MAKE_INT(used_vector_slots),
+			  Fcons(rep_MAKE_INT(rep_used_funargs),
+				rep_MAKE_INT(rep_allocated_funargs
+					     - rep_used_funargs)));
     }
-
-    return(Qt);
+    else
+	return Qt;
 }
 
 
@@ -1135,7 +1113,7 @@ rep_values_init(void)
 void
 rep_values_kill(void)
 {
-    rep_cons_block *cb = cons_block_chain;
+    rep_cons_block *cb = rep_cons_block_chain;
     rep_vector *v = vector_chain;
     rep_string_block *s = string_block_chain;
     while(cb != NULL)
@@ -1162,7 +1140,7 @@ rep_values_kill(void)
 	rep_free(s);
 	s = nxt;
     }
-    cons_block_chain = NULL;
+    rep_cons_block_chain = NULL;
     vector_chain = NULL;
     string_block_chain = NULL;
 }
@@ -1173,6 +1151,10 @@ rep_values_kill(void)
 void
 rep_dumped_init(char *file)
 {
+#if 1
+    fputs ("dumped startup is currently broken\n", stderr);
+    exit (1);
+#else
     void *dl = rep_open_dl_library (rep_string_dup (file));
     if (dl == 0)
 	fprintf (stderr, "warning: couldn't open dumped filed %s\n", file);
@@ -1194,8 +1176,7 @@ rep_dumped_init(char *file)
 	    Fintern_symbol (rep_VAL(s), rep_void_value);
 	    if (s->value == rep_NULL)
 		s->value = rep_void_value;
-	    if (s->prop_list == rep_NULL)
-		s->prop_list = Qnil;
 	}
     }
+#endif
 }
